unit gfx;

interface

procedure h_line(x1, x2, y, color : word);
procedure v_line(x, y1, y2, color : word);
procedure rect(x1, y1, x2, y2, color : word);

implementation

uses
  crt;

procedure h_line(x1, x2, y, color : word); assembler;
asm
	or	bx, -1			{Mask bits}
	mov	di, x1
	mov	cx, di
	and	cl, $07			{cl = x1 % 8}
	shr	bh, cl			{bh = Left mask}
	shr	di, $03			{di = x1 / 8}

	mov	cx, x2
	push	cx
	not	cx
	and	cl, $07			{cl = 7 - x2 % 8}
	shl	bl, cl			{bl = Right mask}
	pop	cx
	shr	cx, $03			{cx = x2 / 8}
	sub	cx, di			{cx = Width}
	jnz	@@1
	and	bh, bl			{Line fits 1 in byte (AND masks)}
@@1:	mov	ax, y
	shl	ax, $04			{ax = 16 * y}
	add	di, ax			{di = 16 * y + x1 / 8}
	shl	ax, $02			{ax = 64 * y}
	add	di, ax			{di = 80 * y + x1  / 8}
	mov	ax, $a000
	mov	es, ax			{es = Video segment}

	xor	al, al			{Set/Reset Register}
	mov	ah, byte ptr color
	mov	dx, $03ce		{Graphics Controller Index}
	out	dx, ax
	mov	ax, $0f01		{Enable Set/Reset Register}
	out	dx, ax
	mov	ah, bh			{ah = Bit mask}
	mov	al, $08			{Bit Mask Register}
	out	dx, ax
	inc	byte ptr es:[di]	{Write left mask}

	inc	dx			{Graphics Controller Data}
	jcxz	@@3			{Width > 0?}
	inc	di
	dec	cx
	jz	@@2			{--Pixels left > 0?}
	mov	al, $ff			{Bit mask: fill 8 pixels}
	out	dx, al
rep	stosb
@@2:	mov	al, bl			{al = Bit mask}
	out	dx, al
	inc	byte ptr es:[di]	{Write right mask}
@@3:	mov	al, $ff			{Default bit mask}
	out	dx, al
	mov	ax, $0001		{Enable Set/Reset Register}
	dec	dx			{Graphics Controller Index}
	out	dx, ax
end;

procedure v_line(x, y1, y2, color : word); assembler;
asm
	mov	bh, $80			{Mask bit}
	mov	di, x
	mov	cx, di			{cx = x}
	and	cl, $07			{cl = x % 8}
	shr	bh, cl			{bh = bit mask}
	shr	di, $03			{di = x / 8}

	mov	ax, y1
	mov	cx, y2
	sub	cx, ax			{cx = Height}
	shl	ax, $04			{ax = 16 * y1}
	add	di, ax			{di = 16 * y1 + x / 8}
	shl	ax, $02			{ax = 64 * y1}
	add	di, ax			{di = 80 * y1 + x / 8}
	mov	ax, $A000
	mov	es, ax			{es = Video segment}

	mov	dx, $03ce		{Graphics Controller Index}
	mov	ah, byte ptr color
	xor	al, al			{Set/Reset Register}
	out	dx, ax
	mov	ax, $0f01		{Enable Set/Reset Register}
	out	dx, ax
	mov	ah, bh			{ah = Bit mask}
	mov	al, $08			{Bit Mask Register}
	out	dx, ax

@@1:	inc	byte ptr es:[di]	{Write pixel}
	add	di, $0050		{Next scanline}
	dec	cx			{Decrease count}
	jnz	@@1			{Zero?}

	mov	al, $ff			{Default bit mask}
	inc	dx			{Graphics Controller Data}
	out	dx, al
	mov	ax, $0001		{Enable Set/Reset Register}
	dec	dx			{Graphics Controller Index}
	out	dx, ax
end;

procedure rect(x1, y1, x2, y2, color : word);
begin
  h_line(x1, x2, y1, color);
  h_line(x1, x2, y2, color);
  v_line(x1, y1, y2, color);
  v_line(x2, y1, y2, color);
end;

begin
  asm
    mov     ax, $0012
    int     $10
  end;
  directvideo := false;
  windmax := $1d4f
end.