#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Anzahl 'Schlsselwrter'; wird aus "words.dat" gelesen
int num_words;

// 'Schlsselwrter' aus "words.dat";
// wird als [num_words][6] array angelegt
//
// words[i] ist das i/6-te 'Schlsselwort' der Sprache i%6
char** words;

///////////////////////////////////////////////////////////////////////////////
//
// Versuche, filename zu ffnen;
// falls ein Fehler auftritt (z.B. die Datei ist nicht vorhanden),
// beende das Programm mit einer Fehlermeldung
//
///////////////////////////////////////////////////////////////////////////////
FILE* open_file(const char* filename)
{
	FILE* file = fopen(filename, "r");	// Nur-lese Zugriff
	if(!file)
	{
		printf("Datei \"%s\" konnte nicht geffnet werden", filename);
		exit(EXIT_FAILURE);
	}

	return file;
}

///////////////////////////////////////////////////////////////////////////////
//
// Lade 'Schsselwrter' aus filename
//
///////////////////////////////////////////////////////////////////////////////
void get_words(const char* filename)
{
	FILE* file = open_file("words.dat");

	// Lese Anzahl der 'Schlsselwrter' aus der Datei ein
	fscanf(file, "%02d", &num_words);
	words = new char*[num_words];

	char str[0x100];	// Puffer fr fgets()

	for(int i = 0; i < num_words; i++)
	{
		// Lese nchstes 'Schlsselwort' aus der Datei ein
		fgets(str, 0x100, file);

		// Entferne '\n' Zeichen
		str[strlen(str) - 1] = '\0';

		// Kopiere str in words[i]
		words[i] = (char*)malloc(strlen(str));
		strcpy(words[i], str);
	}

	fclose(file);
}

///////////////////////////////////////////////////////////////////////////////
//
// main
//
///////////////////////////////////////////////////////////////////////////////
void main(int argc, char* argv[])
{
	///////////////////////////////////////////////////////////////////////
	//
	// Prfe, ob die Eingabedatei als Parameter bergeben wurde;
	// falls nicht, beende das Programm mit einer Fehlermeldung;
	// falls ja, ffne die Datei
	//
	///////////////////////////////////////////////////////////////////////
	if(argc < 2)
	{
		printf("Erforderlicher Kommandozeilen-Parameter "\
		       "(Name der Eingabedatei) fehlt;\n"\
		       "(das Programm muss von einer DOS "\
		       "Eingabeaufforderung gestartet werden)\n"\
		       "Drcken Sie eine Taste, um das Programm zu beenden");
		getchar();
		exit(EXIT_FAILURE);
	}

	FILE* file = open_file(argv[1]);

	// Lade 'Schlsselwrter' aus "words.dat"
	get_words("words.dat");
	
	printf("Analysiere \"%s\"...", argv[1]);
	
	///////////////////////////////////////////////////////////////////////
	//
	// Zhle die 'Schlsselwrter' jeder Sprache im Text
	//
	// Programmverlauf:
	// - fr alle Zeilen im Text:
	//   - lese die Zeile aus der Datei ein
	//   - fr alle bekannten 'Schlsselwrter':
	//     - kommt das Schlsselwort in der Zeile vor?
	//       - ja: inkrementiere die Anzahl gefundener 'Schlsselwrter'
	//             der zugehrigen Sprache
	//
	// Anmerkung:
	//   Die Funktion strstr() sucht das erste Vorkommen
	//   des jeweiligen 'Schlsselwortes' (words[i])
	//   in der Zeile (str); damit werden weitere
	//   Vorkommen des 'Schlsselwortes' in der Zeile ignoriert
	//
	///////////////////////////////////////////////////////////////////////
	char str[0x100];	// Puffer fr fgets()

	// Anzahl gefundener 'Schlsselwrter' je Sprache	
	unsigned int word_count[] = {0, 0, 0, 0, 0, 0, 1};

	while(fgets(str, 0x100, file))
		for(int i = 0; i < num_words; i++)
			if(strstr(str, words[i]))
				word_count[i % 6]++;

	///////////////////////////////////////////////////////////////////////
	//
	// Ermittle die Sprache mit den meisten 'Schlsselwrtern' im Text;
	// gebe sie als Ergebnis aus
	//
	// Anmerkung:
	//   Falls kein 'Schlsselwort' im Text vorkam,
	//   ist word_count[0..5] == 0, und word_count[6] == 1;
	//   die erkannte Sprache ist dann langs[6], oder "??"
	//
	///////////////////////////////////////////////////////////////////////
	unsigned int max = 0;	// Hchste Anzahl gefundener 'Schlsselwrtern'
	int pos = 0;		// Erkannte Sprache (Position in langs[])

	char* langs[] = {"UK", "DE", "FR", "IT", "ES", "TR", "??"};

	for(int i = 0; i < 7; i++)
		if(word_count[i] > max)
		{
			max = word_count[i];
			pos = i;
		}

	printf("%s", langs[pos]);	// Ausgabe der erkannten Sprache

	// Schliesse die Eingabedatei, und beende das Programm
	fclose(file);
	exit(EXIT_SUCCESS);
}