unit MainU;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, ExtCtrls, ComCtrls, ToolWin;

type
  TForm1 = class(TForm)
    Tabelle: TStringGrid;
    Memo1: TMemo;
    ToolBar1: TToolBar;
    StartButton: TToolButton;
    ToolButton1: TToolButton;
    FilterButton: TToolButton;
    Splitter1: TSplitter;
    ToolButton2: TToolButton;
    OptionenButton: TToolButton;
    ToolButton3: TToolButton;
    LoadButton: TToolButton;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    ToolButton5: TToolButton;
    SaveButton: TToolButton;
    DelTabelle: TToolButton;
    ToolButton8: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure InitFields;
    procedure GetTaenzerPos;
    procedure StartButtonClick(Sender: TObject);
    procedure CalculateJump;
    procedure FilterTabelle;
    procedure SimulateFromPos;
    procedure TabelleSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: String);
    procedure FilterButtonClick(Sender: TObject);
    procedure OptionenButtonClick(Sender: TObject);
    procedure LabelTabelle;
    procedure ClearTabelle;
    procedure ResetGroup;
    procedure LoadButtonClick(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure DelTabelleClick(Sender: TObject);
    procedure TabelleGetEditMask(Sender: TObject; ACol, ARow: Integer;
      var Value: String);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

type
  TTaenzer = Record
    Aktuelles_Feld : Integer; // Feld, auf dem der Tnzer aktuell steht
    Altes_Feld     : Integer; // Feld, auf dem der Tnzer letzten Takt stand
    Gefunden       : Boolean; // TRUE, wenn der Tnzer vorhanden ist
    Gesprungen     : Boolean; // TRUE, wenn der Tnzer im aktuellen Takt
                              // gesprungen ist
  end;

type
  TFeld = Record
    Ziele       : ARRAY OF Integer; // Liste mglicher Zielfelder
    Ziel_Anzahl : Integer; // Anzahl mglicher Ziele
    Besetzt     : BOOLEAN; // TRUE, wenn Feld am Taktbegin besetzt ist
    Verlassen   : BOOLEAN; // TRUE, wenn Feld im aktuellen Takt verlassen wurde
  end;

var
  Form1: TForm1;
  Felder: Array[1..8] OF TFeld; // Menge aller Felder
  Gruppe: Array[1..5] OF TTaenzer; // Menge aller Tnzer
  Aktueller_Takt: Integer; // Aktueller Takt
  RowCounter: Integer; // Anzahl der Tabellenzeilen

implementation

uses OptionsU; // Einbindung des Optionsfensters

{$R *.DFM}

procedure TForm1.LabelTabelle; // Benennt Tabellenzeilen und Spalten
VAR X : Integer;
begin
  Tabelle.Cells[0,0] := 'Takt';
  For X := 1 TO 5 DO Tabelle.Cells[X,0] := 'Tnzer ' + IntToStr(X);
  For X := 1 TO RowCounter DO Tabelle.Cells[0,X] := IntToStr(X-1);
end;

procedure TForm1.FormCreate(Sender: TObject);
VAR X: Integer;
begin
  RowCounter := 22; // Setzt die Tabellenzeilen (1 Zeile berschrift +
                    // Takt 0 + 20 zu simulierende Takte )
  Memo1.Lines.Add('Night.Dance Choreographics Debugger :');
  Memo1.Lines.Add('================================');
  LabelTabelle;
  FOR X := 1 TO 5 DO Gruppe[X].Gefunden := FALSE; // bisher noch keine Tnzer
                                                  // gefunden
  Aktueller_Takt := 0;
  InitFields;
end;

procedure TForm1.SimulateFromPos; // Kommt zum Einsatz, falls die Simulation
                                  // NICHT mit Takt 1 startet
                                  // Die FOR-Schleife durchsucht die Tabelle im
                                  // aktuellen Takt. Sitzt der Tnzer schon
                                  // lnger als diesen Takt auf dem aktuellen
                                  // Feld, kann er im vorherigen Takt nicht
                                  // gesprungen sein und darf somit beim
                                  // nchsten Mal sein Glck versuchen :-)
VAR X : Integer;
begin
  IF Aktueller_Takt = 0 THEN INC(Aktueller_Takt);
  FOR X := 1 TO 5 DO
    IF (Tabelle.Cells[X,Aktueller_Takt] = '') OR
       (Tabelle.Cells[X,Aktueller_Takt] = Tabelle.Cells[X,Aktueller_Takt-1])
    THEN Gruppe[X].Gesprungen := FALSE
    ELSE
      Gruppe[X].Gesprungen := TRUE;
end;

procedure TForm1.InitFields; // Initialisierung der Felder
                             // Die Zielpunkte der Felder werden festgelegt,
                             // ihre Anzahl festgelegt und, da noch keine
                             // Simulation luft, Verlassen auf FALSE gesetzt
begin
  Memo1.Lines.Add('');
  SetLength(Felder[1].Ziele,3);
  Felder[1].Ziele[0] := 2;
  Felder[1].Ziele[1] := 3;
  Felder[1].Ziele[2] := 8;
  Felder[1].Ziel_Anzahl := 3;
  Felder[1].Verlassen := FALSE;
  Memo1.Lines.Add('Feld 1 initialisiert');
  SetLength(Felder[2].Ziele,3);
  Felder[2].Ziele[0] := 1;
  Felder[2].Ziele[1] := 7;
  Felder[2].Ziele[2] := 8;
  Felder[2].Ziel_Anzahl := 3;
  Felder[2].Verlassen := FALSE;
  Memo1.Lines.Add('Feld 2 initialisiert');
  SetLength(Felder[3].Ziele,4);
  Felder[3].Ziele[0] := 1;
  Felder[3].Ziele[1] := 4;
  Felder[3].Ziele[2] := 5;
  Felder[3].Ziele[3] := 8;
  Felder[3].Ziel_Anzahl := 4;
  Felder[3].Verlassen := FALSE;
  Memo1.Lines.Add('Feld 3 initialisiert');
  SetLength(Felder[4].Ziele,3);
  Felder[4].Ziele[0] := 3;
  Felder[4].Ziele[1] := 6;
  Felder[4].Ziele[2] := 8;
  Felder[4].Ziel_Anzahl := 3;
  Felder[4].Verlassen := FALSE;
  Memo1.Lines.Add('Feld 4 initialisiert');
  SetLength(Felder[5].Ziele,3);
  Felder[5].Ziele[0] := 3;
  Felder[5].Ziele[1] := 6;
  Felder[5].Ziele[2] := 8;
  Felder[5].Ziel_Anzahl := 3;
  Felder[5].Verlassen := FALSE;
  Memo1.Lines.Add('Feld 5 initialisiert');
  SetLength(Felder[6].Ziele,3);
  Felder[6].Ziele[0] := 4;
  Felder[6].Ziele[1] := 5;
  Felder[6].Ziele[2] := 8;
  Felder[6].Ziel_Anzahl := 3;
  Felder[6].Verlassen := FALSE;
  Memo1.Lines.Add('Feld 6 initialisiert');
  SetLength(Felder[7].Ziele,2);
  Felder[7].Ziele[0] := 2;
  Felder[7].Ziele[1] := 8;
  Felder[7].Ziel_Anzahl := 2;
  Felder[7].Verlassen := FALSE;
  Memo1.Lines.Add('Feld 7 initialisiert');
  SetLength(Felder[8].Ziele,7);
  Felder[8].Ziele[0] := 1;
  Felder[8].Ziele[1] := 2;
  Felder[8].Ziele[2] := 3;
  Felder[8].Ziele[3] := 4;
  Felder[8].Ziele[4] := 5;
  Felder[8].Ziele[5] := 6;
  Felder[8].Ziele[6] := 7;
  Felder[8].Ziel_Anzahl := 7;
  Felder[8].Verlassen := FALSE;
  Memo1.Lines.Add('Feld 8 initialisiert');
  Memo1.Lines.Add('');
end;

procedure TForm1.GetTaenzerPos; // Die Tabelle wird spaltenweise von unten nach
                                // oben durchsucht. Der jeweils erste so
                                // gefundene Eintrag einer Spalte ist die letzte
                                // bekannte Position des Tnzers.
                                // Der letzte vorhandene Eintrag in der Tabelle
                                // markiert mittels seiner Zeilenposition die
                                // aktuell gesprungenen Takte.
VAR X, Y : Integer;
    Found: BOOLEAN;

begin
  FOR X := 1 TO 5 DO
    BEGIN
      Found := FALSE;
      FOR Y := RowCounter DOWNTO 1 DO
        IF Found = FALSE THEN
          IF Tabelle.Cells[X,Y] <> '' THEN // Falls ein Eintrag vorhanden ist...
            BEGIN
              IF Aktueller_Takt = 0 THEN Aktueller_Takt := Y; // Falls
                                                              // Aktueller_Takt
                                                              // nicht
                                                              // zugewiesen,
                                                              // dann
                                                              // Zuweisung.
                                                              // => so wird nur
                                                              // der letzte
                                                              // Eintrag einer
                                                              // Spalte beachtet
              IF Y > Aktueller_Takt THEN Aktueller_Takt := Y; // Falls in der
                                                              // aktuellen
                                                              // Spalte ein
                                                              // Tnzer weiter
                                                              // unten in der
                                                              // Liste steht,
                                                              // als in den
                                                              // Spalten davor,
                                                              // setzt dieser
                                                              // die Taktzahl
                                                              // neu.
              Memo1.Lines.Add('Tnzer ' + IntToStr(X) + ' gefunden in Takt ' +
                              IntToStr(Y-1));
              Gruppe[X].Aktuelles_Feld := StrToInt(Tabelle.Cells[X,Y]);
              Gruppe[X].Gefunden := TRUE; // Tnzer gefunden
              Memo1.Lines.Add('Aktuelles Feld fr Tnzer ' + IntToStr(X) +
                             ' ist Feld ' + IntToStr(Gruppe[X].Aktuelles_Feld));
              Felder[Gruppe[X].Aktuelles_Feld].Besetzt := True; // Feld wird
                                                                // besetzt
              Memo1.Lines.Add('Feld ' + IntToStr(Gruppe[X].Aktuelles_Feld) +
                             ' ist besetzt');
              Found := TRUE; // Tnzer gefunden => nchste Spalte
              Memo1.Lines.Add('');
            END;
      IF Found = False THEN
        BEGIN
          Memo1.Lines.Add('Tnzer ' + IntToStr(X) + ' nicht gefunden');
          Memo1.Lines.Add('');
        END;
    END;
  Memo1.Lines.Add('Simulation startet mit Takt ' + IntToStr(Aktueller_Takt));
  Memo1.Lines.Add('');
end;

procedure TForm1.CalculateJump; // Sie Sprungmglichkeiten werden berechnet.
                                // Dazu werden mit Hilfe zweier Filter
                                // Mglichkeiten aus der Menge der maximalen
                                // Mglichkeiten ausgesiebt.
VAR X : Integer;
    Menge  : ARRAY OF Integer; // Menge der Mglichkeiten
    Length : Integer; // Lnge des Mengen-Arrays
    RZiel  : Integer; // Zufallsauswahl bei mehreren Zielen

  procedure Filter1(Pos:Integer); // Verkleinert die Menge der Maximalziele um
                                  // die Anzahl der schon besetzten Ziele.
  VAR X : Integer;
  BEGIN
    X := 0;
    Length := 0;
    SetLength(Menge,Length); // Menge wird auf 0 gesetzt
    Repeat
      IF NOT (Felder[Pos].Ziele[X] = -1) THEN // Falls ein Feld verfgbar ist...
        IF Felder[Felder[Pos].Ziele[X]].Besetzt = FALSE THEN // und nicht
                                                             // besetzt...
          BEGIN
            INC(Length);
            SetLength(Menge,Length); // Menge wird vergrert
            Menge[Length-1] := Felder[Pos].Ziele[X]; // Das Feld wird der
                                                     // Zielmenge hinzugefgt.
          END;
      INC(X);
    UNTIL X = Felder[Pos].Ziel_Anzahl; // Bricht ab, wenn alle Mglichkeiten
                                       // abgearbeitet sind.
    IF Length > 0 THEN
      BEGIN
        X := Low(Menge);
        REPEAT
          Memo1.Lines.Add('Sprungmglichkeiten von Feld '+IntToStr(Pos)+': ' +
                         IntToStr(Menge[X]));
          INC(X);
        UNTIL X = Length;
      END
    ELSE
      BEGIN
        Memo1.Lines.Add('Keine Sprungmglichkeiten von Feld '+IntToStr(Pos));
        Memo1.Lines.Add('');
      END;
  END;

  procedure Filter2(Start, Ziel : Integer); // Filtert durch Kreuzsprnge
                                            // gesperrte Ziele aus der noch
                                            // verbliebenen Menge heraus.
                                            // Die entsprechenden Ziele werden
                                            // auf -1 gesetzt.
  begin
    case Start of // Start = Start des Sprungs
    1 : case Ziel of // Ziel = Ziel des Sprungs
        3 : begin
              Memo1.Lines.Add('Sprung von Feld 2 nach ' +
                             IntToStr(Felder[2].Ziele[1]) + ' wird gesperrt');
              Felder[2].Ziele[1] := -1;
              Memo1.Lines.Add('Sprung von Feld 2 nach ' +
                             IntToStr(Felder[2].Ziele[2]) + ' wird gesperrt');
              Felder[2].Ziele[2] := -1;
              Memo1.Lines.Add('Sprung von Feld 7 nach ' +
                             IntToStr(Felder[7].Ziele[0]) + ' wird gesperrt');
              Felder[7].Ziele[0] := -1;
              Memo1.Lines.Add('Sprung von Feld 8 nach ' +
                             IntToStr(Felder[8].Ziele[1]) + ' wird gesperrt');
              Felder[8].Ziele[1] := -1;
            end;
        8 : begin
              Memo1.Lines.Add('Sprung von Feld 2 nach ' +
                             IntToStr(Felder[2].Ziele[1]) + ' wird gesperrt');
              Felder[2].Ziele[1] := -1;
              Memo1.Lines.Add('Sprung von Feld 7 nach ' +
                             IntToStr(Felder[7].Ziele[0]) + ' wird gesperrt');
              Felder[7].Ziele[0] := -1;
            end;
        end;
    2 : case Ziel of
        7 : begin
              Memo1.Lines.Add('Sprung von Feld 1 nach ' +
                             IntToStr(Felder[1].Ziele[1]) + ' wird gesperrt');
              Felder[1].Ziele[1] := -1;
              Memo1.Lines.Add('Sprung von Feld 1 nach ' +
                             IntToStr(Felder[1].Ziele[2]) + ' wird gesperrt');
              Felder[1].Ziele[2] := -1;
              Memo1.Lines.Add('Sprung von Feld 3 nach ' +
                             IntToStr(Felder[3].Ziele[0]) + ' wird gesperrt');
              Felder[3].Ziele[0] := -1;
              Memo1.Lines.Add('Sprung von Feld 8 nach ' +
                             IntToStr(Felder[8].Ziele[0]) + ' wird gesperrt');
              Felder[8].Ziele[0] := -1;
            end;
        8 : begin
              Memo1.Lines.Add('Sprung von Feld 1 nach ' +
                             IntToStr(Felder[1].Ziele[1]) + ' wird gesperrt');
              Felder[1].Ziele[1] := -1;
              Memo1.Lines.Add('Sprung von Feld 3 nach ' +
                             IntToStr(Felder[3].Ziele[0]) + ' wird gesperrt');
              Felder[3].Ziele[0] := -1;
            end;
        end;
    3 : case Ziel of
        1 : begin
              Memo1.Lines.Add('Sprung von Feld 2 nach ' +
                             IntToStr(Felder[2].Ziele[1]) + ' wird gesperrt');
              Felder[2].Ziele[1] := -1;
              Memo1.Lines.Add('Sprung von Feld 2 nach ' +
                             IntToStr(Felder[2].Ziele[2]) + ' wird gesperrt');
              Felder[2].Ziele[2] := -1;
              Memo1.Lines.Add('Sprung von Feld 7 nach ' +
                             IntToStr(Felder[7].Ziele[0]) + ' wird gesperrt');
              Felder[7].Ziele[0] := -1;
              Memo1.Lines.Add('Sprung von Feld 8 nach ' +
                             IntToStr(Felder[8].Ziele[1]) + ' wird gesperrt');
              Felder[8].Ziele[1] := -1;
            end;
        5 : begin
              Memo1.Lines.Add('Sprung von Feld 4 nach ' +
                             IntToStr(Felder[4].Ziele[1]) + ' wird gesperrt');
              Felder[4].Ziele[1] := -1;
              Memo1.Lines.Add('Sprung von Feld 4 nach ' +
                             IntToStr(Felder[4].Ziele[2]) + ' wird gesperrt');
              Felder[4].Ziele[2] := -1;
              Memo1.Lines.Add('Sprung von Feld 6 nach ' +
                             IntToStr(Felder[6].Ziele[0]) + ' wird gesperrt');
              Felder[6].Ziele[0] := -1;
              Memo1.Lines.Add('Sprung von Feld 8 nach ' +
                             IntToStr(Felder[8].Ziele[3]) + ' wird gesperrt');
              Felder[8].Ziele[3] := -1;
            end;
        end;
    4 : case Ziel of
        6 : begin
              Memo1.Lines.Add('Sprung von Feld 3 nach ' +
                             IntToStr(Felder[3].Ziele[2]) + ' wird gesperrt');
              Felder[3].Ziele[2] := -1;
              Memo1.Lines.Add('Sprung von Feld 5 nach ' +
                             IntToStr(Felder[5].Ziele[0]) + ' wird gesperrt');
              Felder[5].Ziele[0] := -1;
              Memo1.Lines.Add('Sprung von Feld 5 nach ' +
                             IntToStr(Felder[5].Ziele[2]) + ' wird gesperrt');
              Felder[5].Ziele[2] := -1;
              Memo1.Lines.Add('Sprung von Feld 8 nach ' +
                             IntToStr(Felder[8].Ziele[4]) + ' wird gesperrt');
              Felder[8].Ziele[4] := -1;
            end;
        8 : begin
              Memo1.Lines.Add('Sprung von Feld 3 nach ' +
                             IntToStr(Felder[3].Ziele[3]) + ' wird gesperrt');
              Felder[3].Ziele[3] := -1;
              Memo1.Lines.Add('Sprung von Feld 5 nach ' +
                             IntToStr(Felder[5].Ziele[0]) + ' wird gesperrt');
              Felder[5].Ziele[0] := -1;
            end;
        end;
    5 : case Ziel of
        3 : begin
              Memo1.Lines.Add('Sprung von Feld 4 nach ' +
                             IntToStr(Felder[4].Ziele[1]) + ' wird gesperrt');
              Felder[4].Ziele[1] := -1;
              Memo1.Lines.Add('Sprung von Feld 4 nach ' +
                             IntToStr(Felder[4].Ziele[2]) + ' wird gesperrt');
              Felder[4].Ziele[2] := -1;
              Memo1.Lines.Add('Sprung von Feld 6 nach ' +
                             IntToStr(Felder[6].Ziele[0]) + ' wird gesperrt');
              Felder[6].Ziele[0] := -1;
              Memo1.Lines.Add('Sprung von Feld 8 nach ' +
                             IntToStr(Felder[8].Ziele[3]) + ' wird gesperrt');
              Felder[8].Ziele[3] := -1;
            end;
        8 : begin
              Memo1.Lines.Add('Sprung von Feld 4 nach ' +
                             IntToStr(Felder[4].Ziele[1]) + ' wird gesperrt');
              Felder[4].Ziele[1] := -1;
              Memo1.Lines.Add('Sprung von Feld 6 nach ' +
                             IntToStr(Felder[6].Ziele[0]) + ' wird gesperrt');
              Felder[6].Ziele[0] := -1;
             end;
        end;
    6 : case Ziel of
        4 : begin
              Memo1.Lines.Add('Sprung von Feld 3 nach ' +
                             IntToStr(Felder[3].Ziele[2]) + ' wird gesperrt');
              Felder[3].Ziele[2] := -1;
              Memo1.Lines.Add('Sprung von Feld 5 nach ' +
                             IntToStr(Felder[5].Ziele[0]) + ' wird gesperrt');
              Felder[5].Ziele[0] := -1;
              Memo1.Lines.Add('Sprung von Feld 5 nach ' +
                             IntToStr(Felder[5].Ziele[2]) + ' wird gesperrt');
              Felder[5].Ziele[2] := -1;
              Memo1.Lines.Add('Sprung von Feld 8 nach ' +
                             IntToStr(Felder[8].Ziele[4]) + ' wird gesperrt');
              Felder[8].Ziele[4] := -1;
            end;
        end;
    7 : case Ziel of
        2 : begin
              Memo1.Lines.Add('Sprung von Feld 1 nach ' +
                             IntToStr(Felder[1].Ziele[1]) + ' wird gesperrt');
              Felder[1].Ziele[1] := -1;
              Memo1.Lines.Add('Sprung von Feld 1 nach ' +
                             IntToStr(Felder[1].Ziele[2]) + ' wird gesperrt');
              Felder[1].Ziele[2] := -1;
              Memo1.Lines.Add('Sprung von Feld 3 nach ' +
                             IntToStr(Felder[3].Ziele[0]) + ' wird gesperrt');
              Felder[3].Ziele[0] := -1;
              Memo1.Lines.Add('Sprung von Feld 8 nach ' +
                             IntToStr(Felder[8].Ziele[0]) + ' wird gesperrt');
              Felder[8].Ziele[0] := -1;
            end;
        end;
    8 : case Ziel of
        1 : begin
              Memo1.Lines.Add('Sprung von Feld 2 nach ' +
                             IntToStr(Felder[2].Ziele[1]) + ' wird gesperrt');
              Felder[2].Ziele[1] := -1;
              Memo1.Lines.Add('Sprung von Feld 7 nach ' +
                             IntToStr(Felder[7].Ziele[0]) + ' wird gesperrt');
              Felder[7].Ziele[0] := -1;
            end;
        2 : begin
              Memo1.Lines.Add('Sprung von Feld 3 nach ' +
                             IntToStr(Felder[3].Ziele[0]) + ' wird gesperrt');
              Felder[3].Ziele[0] := -1;
              Memo1.Lines.Add('Sprung von Feld 1 nach ' +
                             IntToStr(Felder[1].Ziele[1]) + ' wird gesperrt');
              Felder[3].Ziele[3] := -1;
            end;
        4 : begin
              Memo1.Lines.Add('Sprung von Feld 3 nach ' +
                             IntToStr(Felder[3].Ziele[3]) + ' wird gesperrt');
              Felder[3].Ziele[3] := -1;
              Memo1.Lines.Add('Sprung von Feld 5 nach ' +
                             IntToStr(Felder[5].Ziele[0]) + ' wird gesperrt');
              Felder[5].Ziele[0] := -1;
            end;
        5 : begin
              Memo1.Lines.Add('Sprung von Feld 4 nach ' +
                             IntToStr(Felder[4].Ziele[1]) + ' wird gesperrt');
              Felder[4].Ziele[1] := -1;
              Memo1.Lines.Add('Sprung von Feld 6 nach ' +
                             IntToStr(Felder[6].Ziele[0]) + ' wird gesperrt');
              Felder[6].Ziele[0] := -1;
            end;
        end;
    end;
  end;

begin
  FOR X := 1 TO 5 DO
    IF Gruppe[X].Gefunden THEN // Falls der Tnzer existiert...
      BEGIN
        IF Gruppe[X].Gesprungen = FALSE THEN // und nicht gesprungen ist...
          BEGIN
            Filter1(Gruppe[X].Aktuelles_Feld); // Filter 1 anwenden
            IF Length > 0 THEN // Falls mindestens ein Ziel gefunden wurde...
              BEGIN
                Gruppe[X].Gesprungen := TRUE; // wird der Tnzer springen.
                IF Length > 1 THEN // Falls mehrere Ziele vorhanden sind...
                  BEGIN
                    RZiel := Random(Length); // wird eins zufllig ausgewhlt
                    Gruppe[X].Altes_Feld := Gruppe[X].Aktuelles_Feld;
                    Gruppe[X].Aktuelles_Feld := Menge[RZiel];
                    Memo1.Lines.Add('Tnzer ' + IntToStr(X) + ' springt von ' +
                                   IntToStr(Gruppe[X].Altes_Feld) + ' nach ' +
                                   IntToStr(Gruppe[X].Aktuelles_Feld));
                    Memo1.Lines.Add('Tnzer ' + IntToStr(X) + ' aktualisiert');
                    Felder[Gruppe[X].Aktuelles_Feld].Besetzt := TRUE;
                    Felder[Gruppe[X].Altes_Feld].Verlassen := TRUE;
                    Memo1.Lines.Add('Feld ' + IntToStr(Gruppe[X].Aktuelles_Feld)
                                   + ' aktualisiert');
                    Memo1.Lines.Add('');
                    Filter2(Gruppe[X].Altes_Feld,Gruppe[X].Aktuelles_Feld);
                    // Filter 2 wird angewendet, um durch Kreuzsprung betroffene
                    // Felder fr nachfolgende Springer zu sperren.
                  END
                ELSE // Wenn nur ein Ziel vorhanden ist, springt der Tnzer
                     // dorthin.
                  BEGIN
                    Gruppe[X].Altes_Feld := Gruppe[X].Aktuelles_Feld;
                    Gruppe[X].Aktuelles_Feld := Menge[0];
                    Memo1.Lines.Add('Tnzer ' + IntToStr(X) + ' springt von ' +
                                   IntToStr(Gruppe[X].Altes_Feld) + ' nach ' +
                                   IntToStr(Gruppe[X].Aktuelles_Feld));
                    Memo1.Lines.Add('Tnzer ' + IntToStr(X) + ' aktualisiert');
                    Felder[Gruppe[X].Aktuelles_Feld].Besetzt := TRUE;
                    Felder[Gruppe[X].Altes_Feld].Verlassen := TRUE;
                    Memo1.Lines.Add('Feld ' + IntToStr(Gruppe[X].Aktuelles_Feld)
                                   + ' aktualisiert');
                    Memo1.Lines.Add('');
                    Filter2(Gruppe[X].Altes_Feld,Gruppe[X].Aktuelles_Feld);
                    // s.o.
                  END;
              END;
          END
        ELSE // Falls kein Ziel vorhanden ist, springt der Tnzer nicht.
          BEGIN
            Memo1.Lines.Add('Tnzer ' + IntToStr(X) + ' springt nicht');
            Gruppe[X].Gesprungen := FALSE;
            Memo1.Lines.Add('Tnzer ' + IntToStr(X) + ' aktualisiert');
            Memo1.Lines.Add('');
          END;
      END;
end;

procedure TForm1.FilterTabelle; // Lscht berflssige Eintrge aus der Tabelle
CONST Length : Integer = 1; // Lnge von Cells
VAR X,Y : Integer; // Zhler
    Cells: ARRAY OF ARRAY[1..2] OF Integer; // Speichert die Addressen
                                            // berflssiger Felder
    Store: String; // Ausgabevariable

begin
  Memo1.Lines.Add('');
  Memo1.Lines.Add('Tabelle wird gefiltert');
  Memo1.Lines.Add('');
  Memo1.Lines.Add('Initialisiere Filter');
  For X := 1 TO 5 DO
    For Y := 1 TO RowCounter-2 DO
      BEGIN
        IF Tabelle.Cells[X,Y-1] = Tabelle.Cells[X,Y] THEN // Falls das aktuelle
                                                          // Feld gleich dem
                                                          // Feld des letzten
                                                          // Taktes ist, wird
                                                          // der Eintrag aus der
                                                          // Tabelle entfernt.
          BEGIN
            SetLength(Cells,Length); // Vergrert Cells
            INC(Length);
            Cells[Length-2][1] := X; //
            Cells[Length-2][2] := Y; // Tabellenkoordinaten werden gespeichert
          END;
      END;
  Memo1.Lines.Add('');
  Memo1.Lines.Add('Folgende Werte werden gelscht');
  Store := '';
  For X := 0 TO Length-2 DO
    Store := Store + '(' + IntToStr(Cells[X][1]) + ',' + IntToStr(Cells[X][2]-1)
             + ')';
  Memo1.Lines.Add('');
  Memo1.Lines.Add(Store);
  Memo1.Lines.Add('');
  Memo1.Lines.Add('Lschen berflssiger Werte');
  For X := 0 TO Length-2 DO // Zelleninhalte der in Cells gespeicherten Zellen
                            // werden gelscht
    Tabelle.Cells[Cells[X][1],Cells[X][2]] := '';
end;

procedure TForm1.ClearTabelle;
VAR X,Y : Integer;
begin
  FOR X := 1 TO 5 DO
    FOR Y := 1 TO RowCounter DO Tabelle.Cells[X,Y] := '';
end;

procedure TForm1.ResetGroup; // Dir Tnyer werden zurckgesetzt
VAR X: Integer;
begin
  FOR X := 1 TO 5 DO
    BEGIN
      Gruppe[X].Aktuelles_Feld := 0;
      Gruppe[X].Altes_Feld := 0;
      Gruppe[X].Gefunden := FALSE;
      Gruppe[X].Gesprungen := FALSE;
    END;
end;

procedure TForm1.StartButtonClick(Sender: TObject); // Simulation
VAR X: Integer;
begin
  Memo1.Lines.Add('Simulationsbegin');
  Memo1.Lines.Add('==============');
  Memo1.Lines.Add('');
  Randomize; // Generiert Zufallszahlen
  Aktueller_Takt := 0;
  InitFields;
  ResetGroup;
  GetTaenzerPos;
  IF Aktueller_Takt <> 1 THEN SimulateFromPos; // Test, ob die Simulation bei
                                               // Takt 1 oder spter beginnt
  REPEAT
    Memo1.Lines.Add('');
    Memo1.Lines.Add('Takt ' + IntToStr(Aktueller_Takt));
    Memo1.Lines.Add('');
    CalculateJump;
    For X := 1 TO 5 DO
      BEGIN
        IF Felder[Gruppe[X].Altes_Feld].Verlassen = TRUE THEN // Falls das Feld
                                                              // im aktuellen
                                                              // Takt verlassen
                                                              // wurde, wird es
                                                              // fr den
                                                              // nchsten Takt
                                                              // freigegeben.
          BEGIN
            Felder[Gruppe[X].Altes_Feld].Besetzt := FALSE;
            Felder[Gruppe[X].Altes_Feld].Verlassen := FALSE;
            Memo1.Lines.Add('Feld '+IntToStr(Gruppe[X].Altes_Feld)+
                           ' freigegeben');
          END;
        Tabelle.Cells[X,Aktueller_Takt+1] := IntToStr(Gruppe[X].Aktuelles_Feld);
      END;
    Memo1.Lines.Add('');
    InitFields;
    Memo1.Lines.Add('Kreuzungsregeln zurckgesetzt');
    Memo1.Lines.Add('');
    Memo1.Lines.Add('Tabellenupdate');
    Memo1.Update;
    Tabelle.Update;
    INC(Aktueller_Takt); // Beginn des nchsten Taktes
  UNTIL Aktueller_Takt = RowCounter-1; // bis die Tabelle voll ist.
  IF Form2.CheckBox1.Checked THEN FilterTabelle; // Falls AutoFilter aktiviert
                                                 // ist, wird die Tabelle jetzt
                                                 // gefiltert.
  Memo1.Lines.Add('');
  Memo1.Lines.Add('Simulationsende');
  Memo1.Lines.Add('==============');
  StartButton.Enabled := FALSE;
end;

procedure TForm1.TabelleSetEditText(Sender: TObject; ACol, ARow: Integer;
  const Value: String); // Tabelleneingabeberprfung ( nur Zahlen zwischen 1
                        // und 8 sind erlaubt. )
VAR Store : String[1];
begin
  IF (Value > '8') OR (Value < '1') THEN
    begin
      Tabelle.Cells[ACol,ARow] := '';
    end;
  IF NOT (Tabelle.Cells[ACol,ARow] = '') THEN
    IF (StrToInt(Tabelle.Cells[ACol,ARow]) > 8) OR
       (StrToInt(Tabelle.Cells[ACol,ARow]) < 1) THEN
      BEGIN
        Store := Tabelle.Cells[ACol,ARow];
        Tabelle.Cells[ACol,ARow] := Store;
      END;
end;

procedure TForm1.FilterButtonClick(Sender: TObject);
begin
  FilterTabelle; // Tabellenausgabe filtern
end;

procedure TForm1.OptionenButtonClick(Sender: TObject);
begin
  Form2.Show; // Optionen anzeigen
end;

procedure TForm1.LoadButtonClick(Sender: TObject);
var Path: String;
    F: TextFile;
    X,Y,Z : Integer;
    Gelesen: STRING;

begin
  ClearTabelle;
  InitFields;
  IF OpenDialog1.Execute THEN
    BEGIN
      Path := OpenDialog1.Filename;
      IF Path <> '' THEN
        BEGIN
          AssignFile(F,Path);
          Reset(F);
          Readln(F,Gelesen);
          RowCounter := StrToInt(Gelesen);
          LabelTabelle;
          Tabelle.RowCount := RowCounter;
          Readln(F,Gelesen);
          Aktueller_Takt := StrToInt(Gelesen);
          X := 1;
          Y := 1;
          FOR Z := 1 TO (Aktueller_Takt*5) DO
            BEGIN
              Readln(F,Gelesen);
              Tabelle.Cells[X,Y] := Gelesen;
              INC(Y);
              IF Y > Aktueller_Takt THEN
                BEGIN
                  INC(X);
                  Y := 1;
                END;
            END;
          Readln(F,Gelesen);
          IF Gelesen = 'TRUE' THEN StartButton.Enabled := TRUE
          ELSE StartButton.Enabled := FALSE;
          CloseFile(F);
        END;
    END;
end;

procedure TForm1.SaveButtonClick(Sender: TObject);
var Path: String;
    F: TextFile;
    X,Y : Integer;

begin
  Aktueller_Takt := 0;
  IF SaveDialog1.Execute THEN
    BEGIN
      Path := SaveDialog1.FileName;
      IF Path <> '' THEN
        BEGIN
          GetTaenzerPos;
          AssignFile(F,Path);
          Rewrite(F);
          Writeln(F,RowCounter);
          Writeln(F,Aktueller_Takt);
          FOR X := 1 TO 5 DO
            FOR Y := 1 TO Aktueller_Takt DO
              Writeln(F,Tabelle.Cells[X,Y]);
          IF StartButton.Enabled = TRUE THEN Writeln(F,'TRUE')
          ELSE Writeln(F,'FALSE');
          Closefile(F);
        END;
    END;
end;

procedure TForm1.DelTabelleClick(Sender: TObject);
begin
  ClearTabelle;
end;

procedure TForm1.TabelleGetEditMask(Sender: TObject; ACol, ARow: Integer;
  var Value: String);
begin
value:='0;1; ';
end;

end.
