(******************************************************************************)
(*                     BWINF 2000 A2 - FINAL VERSION                          *)
(*    ---------------------------------------------------------------         *)
(*      Lsungsbeispiel zu Aufgabe 2 des Bundeswettbewerbs Informatik         *)
(*                                                                            *)
(*    Diese Unit enthlt das Objekt, das alle bentigten Methoden und         *)
(*    Eigenschaften fr das Hauptprogramm kapselt.                            *)
(*                                                                            *)
(*    Copyleft by  Sebastian Hfer                    ( Oktober 2000 )        *)
(*                                                                            *)
(******************************************************************************)

unit bwinf_a2;

interface

uses sysutils;

const
  Spielfeldgroesse = 4; // Radius des "Spielfeldes"
  AnzFelder = 61;       // Anzahl der Felder (Abhngig von der SPIELFELDGROESSE !!)
  max_wert =  5;        // Maximaler Wert beim Fllen mit Zufallswerten

  KEINWEG = -3;
  WEGGEFUNDEN = -1;
  ERREICHT = -2;

type
  TRichtung = (NO, O, SO, SW, W, NW);

  PFeld = ^TFeld;                      // Zeiger auf ein "Spielfeld"

  TVerbindungen = array[TRichtung] of PFeld;  // array mit 6 Zeigern auf "Spielfelder"

  TFeld = record                         // enthlt die Daten fr ein "Spielfeld"
            Nachbar : TVerbindungen;     // Zeiger auf alle Nachbarn des Feldes
            nach : byte;                 // Richtungen, in die Sprnge gingen
            weite : byte;                // Sprungweite von diesem Feld aus
          end;

  TFeldArray = array[0..AnzFelder] of byte;

  TCallback = procedure ( w: integer; r: TRichtung) of object;

  TSpielbrett = class
    Ausgabe: TCallback;
    root : PFeld;
  private
    procedure NeuesFeld(var f: PFeld);
    procedure Feld_Aufbau(const start: PFeld; groesse:word);
    procedure Alle_Felder(var Feld: TFeldarray;const aktion: byte);
    function Rekurs_Suche(const q:integer; R: TRichtung; Von:PFeld): byte;
  public
    constructor create(const aus: TCallback);
    destructor destroy; override;
    procedure Clear;
    procedure Zufallsfuellung;
    procedure Starte_Suche;
    function LoadfromFile(const Filename: string): boolean;
    function SavetoFile(const Filename: string): boolean;
  end;


implementation

uses windows;

const
   RBit : array [TRichtung] of byte = (1, 2, 4, 8, 16, 32);
   (* den Richtungen werden Bitmasken zugewiesen *)
   (* Wird bei der "Ringschluss"-Erkennung benutzt !*)
   // kann man auch durch ( 1 shl ord(<TRichtung>) ) ersetzen ! ;-)

   rechts : array [TRichtung] of TRichtung = (O, SO, SW, W, NW, NO);
   links  : array [TRichtung] of TRichtung = (NW, NO, O, SO, SW, W);
   gegen  : array [TRichtung] of TRichtung = (SW, W, NW, NO, O, SO);
   (* Arrays, um einfach die links und rechts danebenliegenden und die *)
   (* gegenberliegende Richtung zu ermitteln *)

   LESEN = 2;            // Felder werden in den Array eingelesen
   SCHREIBEN = 1;        // Array wird in die Felder geschrieben
                         // gleichzeitig werden die Felder "zurueckgesetzt"
   SETBACK = 0;          // die NACH - Felder werden auf 0 gesetzt

var Feldarray : TFeldarray;

procedure TSpielbrett.NeuesFeld(var f: PFeld);
(* Reserviert Speicher und initialisiert alle Variabeln fr ein neues Feld *)
var d : TRichtung;
begin
  New( f );
  for d:= NO to NW do  f^.Nachbar[d] := nil;
  f^.nach := 0; f^.weite := 0;
end;

constructor TSpielbrett.create;
(* Initialisiert das Objekt ! *)
begin
  inherited create;
  Ausgabe := aus;                        // Callbackprocedur zur Ausgabe im Programm
  NeuesFeld( root );                     // ROOT erstellen
  Feld_Aufbau( root, Spielfeldgroesse ); // Feld erzeugen
end;

destructor TSpielbrett.destroy;
(* Lscht alle Felder des "Spielbretts" *)
var nu, next : PFeld;
    R : TRichtung;

 procedure Feld_Dispose( Q : PFeld);
 (* Alle Verbindungen mit den Nachbarfelder werden gelscht und das Feld entfernt *)
 var R : TRichtung;
 begin
   for R := NO to NW do
    if Q^.Nachbar[R] <> nil then Q^.Nachbar[R].Nachbar[gegen[R]] := nil;
   dispose( Q );
 end;

begin
  nu := root;   R := NO;
  while nu^.Nachbar[W] <> nil do nu := nu^.Nachbar[W];
  repeat
    while nu^.Nachbar[R] = nil do R := rechts[R];
    next := nu^.Nachbar[R];
    Feld_Dispose( nu );
    nu := next;
  until next = root;
  dispose( root );
  inherited destroy;
end;

procedure TSpielbrett.Clear;
(* Die Eigenschaft NACH aller Felder wird auf "0" gesetzt *)
begin
  Alle_Felder( Feldarray, SETBACK );
end;

procedure TSpielbrett.Zufallsfuellung;
(* Alle Felder werden mit Zufallswerten gefllt *)
var i : integer;
begin
  for i := 0 to AnzFelder do FeldArray[i]:= random( max_wert )+1;
  Alle_Felder( Feldarray, SCHREIBEN );
end;

procedure TSpielbrett.Alle_Felder(var Feld: TFeldarray;const aktion: byte);
(* Fhrt eine "Funktion" auf alle Felder aus !! ;-)  *)
var R : TRichtung;
    step, i, f : integer;
    q : PFeld;

 Procedure Tu( xyz : PFeld ); (* Die "Funktion" abhngig von AKTION  *)
 begin
   if f > AnzFelder then exit; // Damit nicht ber den array hinaus gelesen wird !
   case aktion of
    LESEN       : Feld[f] := xyz^.weite;
    SCHREIBEN   : begin xyz^.nach := 0; xyz^.weite := Feld[f]; end;
    SETBACK     : xyz^.nach := 0;
   end;
   inc(f);
 end;

begin
  q := root;  f := 0;
  Tu( q );
  for step := 1 to Spielfeldgroesse do begin
    q := q^.Nachbar[W];
    for R := NO to NW do
     for i:= 1 to step do begin
       Tu( q );
       q := q^.Nachbar[R];
     end;
  end;
end;

function TSpielbrett.SavetoFile(const Filename: string):boolean;
(* Schreibt die Sprungweiten der Felder in eine Datei *)
var datei : file of byte;
begin
  Alle_Felder( Feldarray, LESEN);
  result := false;
  Assignfile( datei, Filename );
  try
   rewrite( datei );
   {$I-}
   Blockwrite( datei, Feldarray, sizeof(Feldarray));
   {$I+}
   if ioresult <> 0 then abort;
   result := true;
  finally
   closefile( datei );
  end;
end;

function TSpielbrett.LoadfromFile(const Filename: string): boolean;
(* Liest die Sprungweiten aus einer Datei und schreibt ihn in das Spielfeld *)
var datei : file of byte;
begin
  result := false;
  if fileexists( Filename ) then begin
   Assignfile( datei, Filename );
   try
    reset( datei );
    if Filesize(datei) >= sizeof(Feldarray) then begin
     {$I-}
     Blockread( datei, Feldarray, sizeof(Feldarray));
     {$I+}
     if ioresult <> 0 then abort;
     result := true;
    end;
    Alle_Felder( Feldarray, SCHREIBEN);
   finally
    closefile( datei );
   end;
  end;
end;


procedure TSpielbrett.Feld_Aufbau(const start: PFeld; groesse: word);
(* Alle Felder werden erstellt und untereinander verknpft *)
var hook, alt, neu : PFeld;
    ring, elem : word;       // Zhler fr den Radius und "Elemente"
    R : TRichtung;           // aktuelle Richtung, in der Felder erstellt werden
// ausgehend von  "root" werden bei W beginnend im Uhrzeigersinn
// die Felder erzeugt und alle verbindungen zu bestehenden Feldern erzeugt

 procedure Verbinde(var a, b: PFeld; weg: TRichtung);
 (* Verbindet Feld A in Richtung WEG mit Feld B *)
 begin
   a^.Nachbar[weg]:= b;            // A -> B
   b^.Nachbar[gegen[weg]]:= a;     // B -> A
 end;

begin
  alt := start;
  for ring := 1 to groesse do
   begin // Neuen Ring beginnen
     hook := alt;
     NeuesFeld( alt );
     Verbinde( hook, alt, W );
     for R := NO to NW do // einmal rundherum ! ;-)
      begin
        if ring > 1 then for elem := 1 to ring-1 do
         begin // neues "Element" hinzufgen
            NeuesFeld( neu );
            Verbinde( alt, neu, R );
            Verbinde( hook, neu, links[R] );
            Verbinde( neu, hook^.Nachbar[R], rechts[R] );
            alt := neu;  hook := hook^.Nachbar[R];
         end;
        if hook^.Nachbar[links[R]] = nil then // Ecke eines "Sechsecks" erreicht
         begin // Neue Ecke
           NeuesFeld( neu );
           Verbinde( alt, neu, R );
           Verbinde( hook, neu, links[R] );
           alt := neu;
         end
        else
         begin // Ecke bereits da -> Kreis schliessen
           Verbinde( alt, hook^.Nachbar[links[R]], R );
           alt := alt^.Nachbar[R];
         end;
      end; { for R  }
   end; { for RING }
end;

procedure TSpielbrett.Starte_Suche;
(* Die Suchfunktion fr den "Rundweg" durchs Feld *)
var R: TRichtung;
    gefunden : boolean;
begin
  Clear;   // "Ringschlusserkennung" zuruecksetzen !
  gefunden := false;
  for R := NO to NW do begin
   if Rekurs_Suche(1, R, root) = 0 then begin
    Ausgabe(WEGGEFUNDEN, NO); gefunden:= true; break; end
  end;
  if not gefunden then Ausgabe(KEINWEG, NO);
end;

function TSpielbrett.Rekurs_Suche(const q:integer; R: TRichtung; Von: PFeld): byte;
(* Der Rekursive Teil der Suchfunktion *)
var p : PFeld;
    i : byte;

 procedure PutOut;   (* Gibt den Text fr einen Sprung aus *)
 begin
   result:=0; // Suche erfolgreich !  :-)
   Ausgabe(von^.weite ,R);
 end;

begin
  result:= 1;
  if (von^.nach and RBit[R]) <> 0 then exit;  // Test ob Sprung schon mal vorkam
  von^.nach := von^.nach or RBit[R];          // Richtung markieren
  // Sprung ausfhren ...
  p := von;
  for i := 1 to von^.weite do
   begin
     p:= p^.Nachbar[R];
     if p = nil then exit;   // Sprung geht "ber den Rand"
   end;
  // Test ob ROOT erreicht
  if p = root then begin Ausgabe(ERREICHT, NO); PutOut; exit end;
  // Weitere Spruenge ...
  if Rekurs_Suche( q+1, links[R], p ) = 0 then begin PutOut; exit; end;
  if Rekurs_Suche( q+1, R, p ) = 0 then begin PutOut; exit; end;
  if Rekurs_Suche( q+1, rechts[R], p ) = 0 then begin PutOut; exit; end;
end;

{ ENDE - BWINF_A2.PAS }
end.

