(******************************************************************************)
(*                     BWINF 2000 A2 - FINAL VERSION                          *)
(*    ---------------------------------------------------------------         *)
(*      Lsungsbeispiel zu Aufgabe 2 des Bundeswettbewerbs Informatik         *)
(*                                                                            *)
(*    Diese Unit enthlt nur den "Feld-Explorer", mit dem der Inhalt der      *)
(*    einzelnen Felder kontrolliert werden kann !                             *)
(*                                                                            *)
(*    Copyleft by  Sebastian Hfer                    ( Oktober 2000 )        *)
(*                                                                            *)
(******************************************************************************)

unit Explor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, BWInf_A2, ImgList;

type
  Tfexplor = class(TForm)
    Image: TImage;
    buts: TImageList;
    nrs: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  public
    proot : PFeld;
    xa, ya, indexa : integer;
    procedure DrawIt;
    procedure DrawLine( w: integer; R: TRichtung);
    procedure AddJump(const w: integer; R: TRichtung);
    procedure ClearJumps;
    procedure DrawJump( index : integer );
  private
    procedure CalcTable;
  end;

var fexplor: Tfexplor;

implementation

{$R *.DFM}

type Tmove = record x,y : integer end;
     PPos = ^TPos;
     TPos = TMove;

var JmpTable : array[TRichtung] of TMove;
    JmpList : TList;

procedure Tfexplor.FormCreate(Sender: TObject);
begin
  CalcTable;  // Sprungtabelle berechnen
  JmpList := TList.Create;  // SprungListe initialisieren
  indexa := 0;
end;

procedure TFexplor.CalcTable;
(* Berechnet aus Richtung und Weite eine Tabelle mit neuen Koordinaten *)
const dist = 36;  // Abstand der Kreismitten
var R : TRichtung;
    w : real;
begin
  w := (1/6)*pi;
  for R := NO to NW do begin
     JmpTable[R].x := round( dist * sin(w) );
     JmpTable[R].y := -round( dist * cos(w) );
     w := w + (1/3)*pi;
  end;
end;

procedure Tfexplor.DrawIt;
(* Zeichnet das Feld im "FeldExplorer" *)
(* Abwandlung von ALLE_FELDER aus BWINF_A2.PAS ! *)
var R : TRichtung;
    step, i, x, y : integer;
    q : PFeld;

 Procedure PutDot(ind: integer);
 begin
   buts.draw(image.canvas, x, y, ind);
   nrs.draw(image.canvas, x+8, y+8, q^.weite );
 end;

begin
  q := proot; x := (image.width div 2)-16; y := (image.height div 2)-16;
  xa := x+16; ya := y+16;       // Koordinaten fr den Mittelpunkt
  with image.Canvas do begin    // Altes Bild lschen ...
   Pen.Width := 2; pen.color := clGray; Brush.Style := bsSolid;
   Fillrect(Rect(0,0,image.width,image.height));
  end;
  PutDot(1);
  for step := 1 to Spielfeldgroesse do begin
    q := q^.Nachbar[W]; x:= x + jmptable[W].x; y := y + jmptable[W].y;
    for R := NO to NW do
     for i:= 1 to step do begin
       PutDot(0);
       q := q^.Nachbar[R]; x:= x + jmptable[R].x; y := y + jmptable[R].y;
     end;
  end;
  Image.Refresh;
end;

procedure TFExplor.DrawLine( w: integer; R: TRichtung);
(* Zeichnet die Linien fr die Sprnge *)
begin
  with image.Canvas do begin
   Pen.Color := clYellow;
   MoveTo( xa, ya );
   xa := xa - w * jmptable[R].x;  ya := ya - w * jmptable[R].y;
   image.Canvas.LineTo( xa, ya );
   Pen.Color := clGreen;
   Ellipse(xa-5,ya-5,xa+5,ya+5);
  end;
end;

procedure TFExplor.AddJump(const  w: integer; R: TRichtung);
var J, A : PPos;   (* Eintrag hinzufgen *)
begin
  New( J );
  with JmpList do begin
   if count <= 0 then begin  // wenn erster sprung ...
    J^.x := image.width div 2; J^.y := image.height div 2; end
   else
    begin // .. sonst ...
     A := items[0];
     J^.x := A^.x - w*JmpTable[R].x;  J^.y := A^.y - w*JmpTable[R].y;
    end;
   Insert( 0, J );
  end;
end;

procedure TFExplor.ClearJumps; (* Alle Eintrge in der Liste lschen *)
var i : integer;
    J : PPos;
begin
  for i:=JmpList.Count-1 downto 0 do
   begin
     J := JmpList.Items[i];
     JmpList.Delete(i);
     dispose( J );
   end;
end;

procedure TFExplor.DrawJump( index : integer );
var A : PPos;

 procedure Circelum(const x ,y : integer );
 const k = 18; begin image.canvas.Ellipse( x-k, y-k, x+k+1, y+k+1 ); end;

 procedure malma(const i : integer; weg : boolean );
 begin
   if not weg then Image.Canvas.Pen.Color := clRed;
   if (i = 0) or (i = JmpList.Count) then begin
    A := JmpList.Items[0];
    Circelum( A^.x, A^.y ); end
   else begin
    A := JmpList.Items[i];
    Circelum( A^.x, A^.y );
    A := JmpList.Items[i-1];
    if not weg then Image.Canvas.Pen.Color := clDkGray;
    Circelum( A^.x, A^.y );
  end;
 end;

begin
  Image.Canvas.Pen.Color := clsilver;
  Image.Canvas.brush.Style := bsClear;
  malma( indexa, true );
  indexa := index ;
  malma( index, false );
end;

procedure Tfexplor.FormDestroy(Sender: TObject);
begin
  ClearJumps;          // Liste Lschen
  JmpList.Destroy;     // Liste aus Speicher entfernen
end;

end.



