// Dokumentindexgenerator v1.1, 8/4/2001
// Jan Wassenberg
program index_gen;

uses
	sysutils, classes;

const
	letters = ['a'..'z', 'A'..'Z'];
	vowels  = ['a', 'e', 'i', 'o', 'u', 'y'];
	consonants = letters-vowels;

var
	end_list, replace_list, ignore_list, index_list : tstringlist;
	prereqs : array of tstringlist;

	f : textfile; // Eingabedatei

	word : string; // Das Wort, das gerade bearbeitet wird
	dummy : integer; // Rckgabewert von TStringList.Find [wird ignoriert]


//---------------------------------------------------------------------------------------------------------
// ffne die Eingabedatei, lade die Ersetzungsregeln und initialisiere die Wortliste
procedure init;
var
	i : integer;
begin
	try
        // Beende das Programm mit einer Fehlermeldung, falls nicht gengend Parameter bergeben wurden
		if paramcount <> 2 then
        	raise exception.create('Aufruf: index_gen text.txt text.ndx, oder ber index_gen.bat');

		// ffne die Eingabedatei
		assign(f, paramstr(1));
		reset(f);

		// Lade Wortendungsstrings
		end_list := tstringlist.create;
		end_list.loadfromfile('rules\end.txt');

		// Lade Ersetzungsstrings
		replace_list := tstringlist.create;
		replace_list.loadfromfile('rules\replace.txt');

		// Stelle sicher, dass Anzahl Endungen = Anzahl Ersetzungen
		if end_list.count <> replace_list.count then
			raise exception.create('Anzahl Endungen nicht gleich Anzahl Ersetzungen');

		// Lade Voraussetzungen
		setlength(prereqs, end_list.count);
		for i := 0 to end_list.count - 1 do begin
			prereqs[i] := tstringlist.create;
			prereqs[i].loadfromfile('rules\prereq' + inttostr(i) + '.txt');

			// Falls eine Voraussetzungsdatei leer ist, beende das Programm
            if prereqs[i].count = 0 then
            	raise exception.create('Leere Voraussetzungsdatei');
		end; // of for

		// Lade Stoppwortliste
		ignore_list := tstringlist.create;
		ignore_list.loadfromfile('rules\ignore.txt');
		ignore_list.sort; // .Find() wird verwendet, also muss die Liste sortiert werden

		// Initialisiere Wortliste
		index_list := tstringlist.create;
		index_list.sorted := true; 			// Wortliste soll sortiert werden,
		index_list.duplicates := dupignore; // damit doppelte Wrter ignoriert werden
	except
		on e:exception do begin
			writeln('Fehler bei der Initialisierung:');
			writeln('  ', e.message);
			writeln('Drcken Sie die Eingabetaste, um das Programm zu beenden');
			readln;
			halt(1);
		end; // of on
	end; // of except
end;


//---------------------------------------------------------------------------------------------------------
// Ermittle das nchste Wort aus der Eingabedatei;
// Wrter werden durch ' ' oder nicht-Buchstaben getrennt.
// Wenn das Dateiende erreicht wird, wird '' zurckgegeben.
function get_next_word : string;
var
	temp_char : char;
begin
	// Alle Zeichen bis zum ersten Buchstaben ignorieren
	repeat read(f, temp_char); until eof(f) or (temp_char in letters);

	// Alle Buchstaben bis zum nchsten nicht-Buchstaben in result kopieren
	result := '';
	while not eof(f) and (temp_char in letters+['''']) do begin
		result := result + lowercase(temp_char);
		read(f, temp_char);
	end; // of while
end;


//---------------------------------------------------------------------------------------------------------
// Ersetzt das Wortende, sofern es eine dafr eine Ersetzungsregel gibt,
// und die dafr erforderliche Vorraussetzung erfllt ist.
// Falls eine Ersetzung durchgefhrt wurde, wird true zurckgegeben;
// eine weitere Ersetzung kann evtl. gemacht werden.
function strip_ending(var word : string) : boolean;
var
	i, j, k : integer;
	end_len, end_pos : integer; // Lnge des zu ersetzenden Wortendes bzw. dessen Anfang in word
begin
	// Prfe, ob word mit einer Wortendung aus end_list endet 
	for i := 0 to end_list.count-1 do begin
		end_len := length(end_list[i]);	  	// Lnge des zu ersetzenden Wortendes
		end_pos := length(word)-end_len+1;	// Stelle im Wort, wo die Endung anfngt

		// Wortende = end_list[i]?
		if copy(word, end_pos, end_len) = end_list[i] then
			// Ja - prfe, ob ein Voraussetzungsstring aus prereqs[i] dem Ersetzungsstring vorausgeht
			for j := 0 to prereqs[i].count-1 do
            	// Buchstaben der Voraussetzung mit denen des Wortendes vergleichen
				for k := end_pos-length(prereqs[i][j]) to end_pos-1 do begin
					case prereqs[i][j][end_pos-k] of
						'?':; // Wildcard: immer gleich <==> Voraussetzung immer erfllt
						'%': if not (word[k] in vowels) then break;				// falls nicht gleich,
						'*': if not (word[k] in consonants) then break;			// gehe zur nchsten
						else if word[k] <> prereqs[i][j][end_pos-k] then break;	// Voraussetzung
					end; // of case

					// Alle Buchstaben im Voraussetzungsstring wurden gefunden; ersetze das Wortende
					if k = end_pos-1 then begin
						delete(word, end_pos, end_len);
						word := word + replace_list[i];
						result := true; // Weitere Ersetzungen knnen evtl. gemacht werden
						exit;
                    end; // of if
				end; // of for k
    end; // of for i

	result := false; // Keine Ersetzungsmglichkeit gefunden;
    				 // das Wort ist auf den Wortstamm zurckgefhrt 
end;


//---------------------------------------------------------------------------------------------------------
begin
	// ffne Eingabedatei und Ersetzungsregeln
	init;

	repeat
		// Hole das nchste Wort aus der Eingabedatei
		word := get_next_word;
		if word = '' then break; // Dateiende erreicht; beende die Schleife

		// Ist das aktuelle Wort ein 'Stoppwort'? Falls ja, berspringe das Wort
		if ignore_list.find(word, dummy) then continue;

		// Entferne das Wortende
		while strip_ending(word) do;

		// Fge das aktuelle Wort der Wortliste zu; Wortwiederholungen werden ignoriert
		index_list.add(word);
	until false;

	// Schliesse die Eingabedatei und speichere die Wortliste in der Ausgabedatei
	closefile(f);
	index_list.savetofile(paramstr(2));
end.
