// night.dance v2.5, 23/04/2001
// Jan Wassenberg
//---------------------------------------------------------------------------

#include <vcl.h>
#include "main.h"
#pragma hdrstop
#include <math.h>
#include <stdio.h>
#include <assert.h>
//---------------------------------------------------------------------------

#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;

#pragma warn -lvc // Unterdrckt Warnung W8030 bei Find(x, NULL)
#pragma warn -com // Unterdrckt Warnung W8010 bei // ... \
//---------------------------------------------------------------------------

// Programmzustnde
enum
{
	RECORD,
	RUNNING,
	STOP
}
state = RECORD;
bool PLAYBACK = false;


// Zustnde fr ein leeres Feld:
const int EMPTY		= -1;
const int JUST_JUMPED	= -2;

// Farbe der Tnzer
const TColor dancer_colors[MAX_DANCERS] = {clRed, clGreen, clYellow, clTeal};


// Arten der Visualisierung (siehe ViewMenu)
#define NO_SHOW (ViewButton->Tag == 0)
#define SHOW_ALGO (ViewButton->Tag == 1)
#define SHOW_STRUCT (ViewButton->Tag == 2)


// Alle Pfeile auf dem Tanzfeld lschen
#define CLEAR_FIELD Field->Picture->Bitmap->Assign(jpeg);

// Verzgerung um ms Millisekunden
#define DELAY(ms)\
{\
	Application->ProcessMessages();\
	Sleep(ms);\
	Application->ProcessMessages();\
}

// Sicherstellen, dass dancer, pos und dest gltig sind
#define ASSERT_VALID(dancer, pos, dest)\
if(dancer < 0 || dancer >= MAX_DANCERS || pos < 0 || pos >= MAX_FIELDS || dest < 0 || dest >= MAX_FIELDS)\
	throw Exception(Format("Ungltige Position/Sprung: %d:%d->%d", ARRAYOFCONST((dancer, pos, dest))));
//---------------------------------------------------------------------------



// Einmalige Initialisierung: lade Daten, zeichne das Feld und verknpfe images[]
void __fastcall TForm1::FormCreate(TObject *Sender)
{
	randomize();

	Application->OnIdle = idle;
	Application->OnException = error;

	// Feld-Daten lesen
	// Hinweis: alle file handles werden mit _fcloseall() freigegeben
	FILE* f = fopen("data\\field.txt", "r");
	fscanf(f, "%d\n%d\n", &num_fields, &num_intersects);

	// Programm mit Fehlermeldung beenden, falls es zuviele Felder in der Eingabedatei gibt
	if(num_fields > MAX_FIELDS || num_intersects > MAX_INTS)
		throw Exception("field.txt: zu viele Felder oder Intersects");

	// Intersect-Locks laden
	for(int i = 0; i < num_intersects; i++)
	{
		intersects[i] = new TStringList;
		intersects[i]->LoadFromFile("data\\int"+IntToStr(i)+".txt");
		intersects[i]->Sort();
		intersect_avail[i] = true;
	}

	// Feldmittelpunkte einlesen
	for(int i = 0; i < num_fields; i++)
		if(fscanf(f, "%d,%d\n", &field_centers[i].x, &field_centers[i].y) != 2)
			throw Exception("field.txt: Fehler bei Feldmittelpunkt #"+IntToStr(i));

	// Feldverbindungen laden und zeichnen
	for(int i = 0; i < num_fields; i++)
	{
		// Verbindungen einlesen
		connections[i] = new TStringList;
		connections[i]->LoadFromFile("data\\con"+IntToStr(i)+".txt");
		connections[i]->Sort();

		// Verbindungen zeichnen
		for(int j = 0; j < connections[i]->Count; j++)
		{
			int dest = connections[i]->Strings[j].ToInt();
			Field->Canvas->MoveTo(field_centers[i].x, field_centers[i].y);
			Field->Canvas->LineTo(field_centers[dest].x, field_centers[dest].y);
		}
	}

	// Felder zeichnen
	Graphics::TBitmap* bmp = new Graphics::TBitmap;
	bmp->LoadFromFile("data\\blue_field.bmp");
	bmp->Transparent = true;
	for(int i = 0; i < num_fields; i++)
		Field->Canvas->Draw(field_centers[i].x-32, field_centers[i].y-32, bmp);
	delete bmp;

	// Fertig gezeichnetes Tanzfeld speichern
	jpeg = new TJPEGImage();
	jpeg->Assign(Field->Picture->Bitmap);
	jpeg->SaveToFile("data\\field.jpg");

	// Datenstrukturen initialisieren:
	temp = new TStringList;
	chor_data = new TStringList;

	images[0] = Image0; images[1] = Image1; images[2] = Image2; images[3] = Image3;

	// Felder initialisieren
	for(int i = 0; i < num_fields; i++)
		fields[i] = EMPTY;

	// Tnzer initialisieren
	for(int i = 0; i < MAX_DANCERS; i++)
		dancer_pos[i] = -1;
}
//---------------------------------------------------------------------------


// Verfgbarkeit der Toolbarknpfe bestimmen
void __fastcall TForm1::idle(TObject *Sender, bool &Done)
{
	// Alle Knpfe ausser ViewButton sind nicht erlaubt,
	// falls im manuellen Record-Modus der Takt nicht abgeschlossen ist
	// (also ExecuteButton->Enabled == true)

	// Neu bzw. ffnen zulassen, wenn keine Simulation luft
	NewButton->Enabled  = state == RECORD && !ExecuteButton->Enabled;
	OpenButton->Enabled = state == RECORD && !ExecuteButton->Enabled;

	// Speichern zulassen, wenn keine Simulation luft und es aufgenommene Daten gibt
	SaveButton->Enabled = state == RECORD && rec_data.data_avail() && !ExecuteButton->Enabled;

	// Go zulassen, wenn es Tnzer auf dem Feld gibt
	GoButton->Enabled = num_dancers > 0 && !ExecuteButton->Enabled;
	GoButton->Hint = (rec_data.data_avail())? "Aufgenommene Sprnge abspielen" : "Choreographie erzeugen";
}
//---------------------------------------------------------------------------


// Beendet bei einer Exception das Programm mit einer Fehlermeldung
void __fastcall TForm1::error(TObject *Sender, Exception *E)
{
	Application->ShowException(E);
	Application->Terminate();
}



// Initialisierung fr ein neues "Dokument"
void __fastcall TForm1::NewClick(TObject *Sender)
{
	// Aufgenommene Daten lschen
	rec_data.clear();
	chor_data->Clear();

	clock = 0;

	state = RECORD;
	PLAYBACK = false;

	CLEAR_FIELD;

	Caption = "night.dance";
}
//---------------------------------------------------------------------------

// Eine Choreographie von einer Datei laden und direkt ausfhren
void __fastcall TForm1::OpenClick(TObject *Sender)
{
	if(!OpenDialog->Execute())
		return;

	NewClick(0);
	rec_data.load(OpenDialog->FileName);
	Caption = "night.dance ["+ExtractFileName(OpenDialog->FileName+"]");

	GoButtonClick(0);
}
//---------------------------------------------------------------------------

// Aufgenommene Daten in eine Choreographie speichern
void __fastcall TForm1::SaveClick(TObject *Sender)
{
	if(!SaveDialog->Execute())
		return;

	String fn = SaveDialog->FileName;
	chor_data->SaveToFile(fn);
	rec_data.save(fn+".ndr");

	Caption = "night.dance ["+ExtractFileName(fn)+"]";
}
//---------------------------------------------------------------------------

void __fastcall TForm1::AboutClick(TObject *Sender)
{
	Application->MessageBox("night.dance v2.5, 23/04/2001\nJan Wassenberg", "Info", MB_OK | MB_ICONINFORMATION);
}
//---------------------------------------------------------------------------


void __fastcall TForm1::ViewButtonClick(TObject *Sender)
{
	ViewMenu->Popup(Left+124, Top+48);
}
//---------------------------------------------------------------------------

// Whlt den Visualisierungstyp
void __fastcall TForm1::ViewMenuItemClick(TObject *Sender)
{
	TMenuItem* menu_item = dynamic_cast<TMenuItem*>(Sender);
	ViewButton->Tag = menu_item->Tag;
	menu_item->Checked = true;
	CLEAR_FIELD; // Alle Ausgaben des letzten Visualisierungsmodus lschen
}
//---------------------------------------------------------------------------




// Zeichnet zur Visualisierung des Ablaufs einen Pfeil der Farbe color von Feld pos nach Feld dest.
// Der Pfeil startet und endet am Rand des jeweiligen Feldes,
// damit das Feld selber nicht berdeckt wird.
// Im Algorithmusvisialisierungsmodus zeigt die Krperfarbe vom Pfeil an,
// ob der Zug, den der Pfeil symbolisiert, erlaubt ist:
// clYellow  : Der Sprung ist mglich
// clRed     : Das Zielfeld ist bereits besetzt
// clFuchsia : Der Sprung wrde zu einer "midair collision" fhren
// clGreen   : Der Sprung wird spter ausgefhrt
// Die Farbe der Pfeilspitze identifiziert den springenden Tnzer.
// Im Strukturenvisualisierungsmodus gibt es folgende Pfeile:
// clYellow  : Der Sprung ist Teil eines "loop"s (geschlossener Vektorzug)
// clLime    : Der Sprung  "    "    "      " und wurde bereits ausgefhrt
// clMaroon  : Eine "xchg" Struktur - Tnzer springen entlang diesem Pfeil hin und her
//
// Algorithmus:
// Endpunkte des Pfeils sind die Schnittpunkte des Verbindungsvektors
// v = (dest_x-pos_x, dest_y-pos_y) mit den beiden Feldkreisen.
// Dazu wird ein Vektor (dx, dy) als (v / |v|) * 32 definiert;
// dieser Vektor ist die Verbindung vom Mittelpunkt eines Feldes
// zum Schnittpunkt von v und dem Feldkreis.
// Die Endpunkte des Pfeils sind dann (pos_x+dx | pos_y+dy) und (dest_x-dx | dest_y-dy).
//
// Bild:
//                  v
//         -------------------->
//
//          d                          /----> Feldkreis
//         ===>                       /
//     _______               _______ /
//    /       \             /       \
//    |       |           \\|       |
//    |   x===#-------------#==>x   |
//    |       |           //|       |
//    \_______/             \_______/
//             [ der Pfeil ]
//
// x sind die Feldmittelpunkte,
// # sind die Schnittpunkte von v mit den Feldkreisen (x +/- d)
void __fastcall TForm1::draw_arrow(int dancer, int pos, int dest, TColor color)
{
	// Nur im Visualisierungs- und manuellen Record-Modus Pfeile zeichnen
	if(state != RECORD && NO_SHOW)
		return;

	ASSERT_VALID(dancer, pos, dest);

	TCanvas* canvas = Field->Canvas;
	TPen* pen = canvas->Pen;

	// Mittelpunkt vom Start- bzw. Endfeld ermitteln
	int pos_x  = field_centers[pos ].x; int pos_y  = field_centers[pos ].y;
	int dest_x = field_centers[dest].x; int dest_y = field_centers[dest].y;

	// (dx, dy) ermitteln
	int dx = dest_x - pos_x; int dy = dest_y - pos_y;
	double d_length = sqrt(dx*dx + dy*dy);
	dx *= (32+2)/d_length; dy *= (32+2)/d_length;

	// Pfeilkrper zeichnen
	pen->Color = color; pen->Width = 4;
	canvas->MoveTo(pos_x+dx, pos_y+dy);
	canvas->LineTo(dest_x-dx, dest_y-dy);

	// (ndx, ndy) ist orthogonal zu (dx, dy), und bestimmt die Breite der Pfeilspitzenlinien.
	int ndx = -dy/2; int ndy = dx/2;

	// Pfeilspitze zeichnen
	pen->Color = dancer_colors[dancer]; pen->Width++;
	canvas->LineTo(dest_x-2*dx - ndx, dest_y-2*dy - ndy);
	canvas->MoveTo(dest_x-dx, dest_y-dy);
	canvas->LineTo(dest_x-2*dx + ndx, dest_y-2*dy + ndy);

	// Der Tnzer hat sich auf diesen Sprung festgelegt;
	// markiere das Zielfeld als von dem Tnzer besetzt (mit einem kleinen Kreis).
	if(color == clGreen)
	{
		canvas->Brush->Color = dancer_colors[dancer];
		canvas->Ellipse(dest_x-16, dest_y-16, dest_x+16, dest_y+16);
	}
}
//---------------------------------------------------------------------------


// Zeichnet einen String in der Nhe des Feldes pos - falls mglich, oberhalb,
// ansonsten unterhalb des Feldes.
// Wird zur Ausgabe der Wiederholungszhler von Strukturen verwendet.
void __fastcall TForm1::draw_text(int dancer, int pos, String text)
{
	// Nur im Strukturenvisualisierungsmodus anzeigen
	if(!SHOW_STRUCT)
		return;

	ASSERT_VALID(dancer, pos, 0);

	TCanvas* canvas = Field->Canvas;
	canvas->Brush->Color = clWhite;
	canvas->TextFlags &= ~ETO_OPAQUE;

	TFont* font = canvas->Font;
	font->Size = 16;
	font->Color = dancer_colors[dancer];
	font->Style << fsBold;

	int x = field_centers[pos].x-16;
	if(x < Field->Left + 16)
		x += 32;
	int y = field_centers[pos].y-64;
	if(y < Field->Top + 16)
		y += 128;
	canvas->TextOut(x, y, text);
}
//---------------------------------------------------------------------------


// Gibt eine StringList der mglichen Sprnge von pos aus zurck;
// wenn Visualisierung der Algorithmusentscheidungen aktiviert ist,
// werden die Sprungmglichkeiten entsprechend ihrer Zulssigkeit mit
// Pfeilen dargestellt: Gelb -> mglich, Rot -> Zielfeld besetzt,
// Lila -> midair collision.
// Algorithmus:  
// Alle grundstzlich mglichen Sprnge (d.h. alle Verbindungen bei pos) werden geprft.
// Falls das Zielfeld besetzt ist oder durch den Sprung eine "midair collision" passieren
// wrde, wird der Sprung aus der Ergebnismenge gelscht.
TStringList* __fastcall TForm1::moves_avail(int pos)
{
        // pos ist nicht gltig, also gibt es auch keine Sprungmglichkeiten  
	if(pos < 0 || pos >= MAX_FIELDS)
	{
		temp->Clear();
		return temp;
	}
	int dancer = fields[pos];

	// Mit der Ergebnismenge "alle mglichen Verbindungen" anfangen
	temp->Assign(connections[pos]);

	// Alle Sprungmglichkeiten prfen
	// (volatile int) ist ntig, weil sich temp->Count evtl. in der Schleife ndert
	for(int i = 0; i < (volatile int)temp->Count; i++)
	{
		int dest = temp->Strings[i].ToInt();

		if(SHOW_ALGO || state == RECORD)
			draw_arrow(dancer, pos, dest, clYellow);

		// Falls das Zielfeld besetzt ist, streiche den aktuellen Zug
		if(fields[dest] != EMPTY)
		{
			if(SHOW_ALGO || state == RECORD)
				draw_arrow(dancer, pos, dest, clRed);
			temp->Delete(i--); // i--, weil Delete() alle Elemente nachrckt
		}
		// Im Falle einer "midair collision" streiche den aktuellen Zug
		else
		{ 
			String move = IntToStr(pos)+IntToStr(dest);
			for(int j = 0; j < num_intersects; j++)
				if(intersects[j]->Find(move, NULL) && !intersect_avail[j])
				{
					if(SHOW_ALGO || state == RECORD)
						draw_arrow(dancer, pos, dest, clFuchsia);
					temp->Delete(i--);
					break;
				}
		}

		// Knstliche Verlangsamung bei Visualisierung der Algorithmusentscheidungen
		if(state != RECORD && SHOW_ALGO)
			DELAY(250);
	}

	return temp;
}
//---------------------------------------------------------------------------


// 'Plant' einen Sprung des Tnzers dancer von pos nach dest im aktuellen Takt.
// Der Tnzer legt sich dabei fest; der Sprung wird zusammen mit allen anderen im Takt
// dann ausgefhrt, wenn alle Tnzer ihren Sprung fertig geplant haben.
// Hierzu wird der Sprung in moves gespeichert, Intersect-Locks werden aktualisiert,
// der Tnzerzustand wird gendert, der Sprung wird aufgenommen, falls nicht im
// Playback-Modus, und schliesslich alle 'mglich' Pfeile werden gelscht.
void __fastcall TForm1::plan_jump(int dancer, int pos, int dest)
{
	// Nur bei gltigen Sprngen planen
	// pos == -1, wenn im RUNNING-Zustand ein Tnzer nicht sichtbar ist;
	// dest == -1, wenn "   "        "     "     "     "   springen kann. 
	if(pos != -1 && dest != -1)
	{
		ASSERT_VALID(dancer, pos, dest);

		// Erster gltiger Sprung: Anfangszustand speichern
		if(!rec_data.data_avail())
			for(int i = 0; i < MAX_DANCERS; i++)
				rec_data.pushi(dancer_pos[i]);

		// Erste beiden Choreographiezeilen (Tnzernummern und Anfangszustand) speichern
		if(!chor_data->Count)
		{
			String numbers = "", pos = "0";
			for(int i = 0 ; i < MAX_DANCERS; i++)
			{
				numbers += "\t"+IntToStr(i);
				pos += "\t"+IntToStr(dancer_pos[i]);
			}
			chor_data->Add(numbers);
			chor_data->Add(pos);
		}

		// Sprunginformationen in moves[] speichern
		moves[dancer].valid = true;
		moves[dancer].pos = pos;
		moves[dancer].dest = dest;
		moves[dancer].dx = field_centers[dest].x - field_centers[pos].x;
		moves[dancer].dy = field_centers[dest].y - field_centers[pos].y;

		if(state == RECORD || SHOW_ALGO)
			draw_arrow(dancer, pos, dest, clGreen);

		// Sprung 'aufnehmen', falls nicht im Playback-Modus
		if(!PLAYBACK)
		{
			rec_data.push("xchg");
			rec_data.push(Format("%d %d %d %d", ARRAYOFCONST((dancer, pos, dest, 1))));
		}

		// Falls der aktuelle Sprung eventuell andere Sprnge durch
		// eine "midair collision" verhindern knnte, markiere
		// relevante intersect locks als besetzt
		String move = IntToStr(pos)+IntToStr(dest);
		for(int i = 0; i < num_intersects; i++)
			if(intersects[i]->Find(move, NULL))
				intersect_avail[i] = false;

		// Tnzerposition aktualisieren
		dancer_pos[dancer] = dest;
		fields[pos] = JUST_JUMPED;
		fields[dest] = dancer;
	}

	// Im Info-Modus alle Pfeile ausser den letzten Grnen lschen
	if(state != RECORD && SHOW_ALGO)
	{
		DELAY(500);
		CLEAR_FIELD;
		for(int i = 0; i < MAX_DANCERS; i++)
			if(moves[i].valid)
				draw_arrow(i, moves[i].pos, moves[i].dest, clGreen);
		DELAY(500);
	}
}

// Fhrt einen Sprung von allen Tnzern gleichzeitig aus.
// Zuerst springen die Tnzer auf ihr Zielfeld,
// danach wird der nchste Takt vorbereitet
void __fastcall TForm1::do_jump()
{
	if(!NO_SHOW)
		DELAY(1000);

	// Ende des letztes Taktes "aufnehmen"
	if(!PLAYBACK)
		rec_data.push("end");

	// Choreographiedaten speichern
	String pos = IntToStr(++clock);
	for(int i = 0; i < MAX_DANCERS; i++)
		pos += "\t"+IntToStr(dancer_pos[i]);
	chor_data->Add(pos);

	// Alle Tnzer gleichzeitig springen lassen
	MoveTimer->Tag = 0; MoveTimer->Enabled = true;
	while(MoveTimer->Enabled)
		Application->ProcessMessages();

	// move-Information zurcksetzen und Choreographiedaten speichern
	for(int i = 0; i < MAX_DANCERS; i++)
		moves[i].valid = false;

	// Felder, von der ein Tnzer im letzten Takt abgesprungen war, wieder freigeben
	for(int i = 0; i < num_fields; i++)
		if(fields[i] == JUST_JUMPED)
			fields[i] = EMPTY;

	// Alle Intersect-Locks freigeben
	for(int i = 0; i < num_intersects; i++)
		intersect_avail[i] = true;
}

// Der eingegebener Takt wird ausgefhrt (nur fr manuellen Record-Modus)
void __fastcall TForm1::ExecuteButtonClick(TObject *Sender)
{
	ExecuteButton->Enabled = false;

	do_jump();
}
//---------------------------------------------------------------------------


// Spielt einen gespeicherten Takt im Record-Puffer ab
void __fastcall TForm1::playback()
{
	STRUCT* s;

	// Alle Befehle fr den aktuellen Takt aus dem Record-Puffer lesen
	while(!rec_data.at_end())
	{
                // Aktuellen Befehl aus dem Record-Puffer lesen
		String cmd = rec_data.pop();
		// Taktende-Befehl
		if(cmd == "end")
			break;

		// Sicherstellen, dass dies ein gltiger Befehl ist
		if(cmd != "loop" && cmd != "xchg")
			throw Exception(("Ungltiger Befehl: \""+cmd+"\" (falsche Lnge beim letzten \"loop\"?)"));

		// Parameter fr den aktuellen Befehl aus dem Record-Puffer lesen
		String data = rec_data.pop();

		// Die einstellige Tnzernummer aus dem Befehlsparameter lesen
		assert(MAX_DANCERS < 10);
		int dancer = (int)data[1] - '0'; // AnsiString::operator[] ist 1-basiert!!!
		ASSERT_VALID(dancer, 0, 0);

		// Den Eintrag in exec_buffer, der zu dancer gehrt, ermitteln
		s = &exec_buffer[dancer];
		// Hat dieser Tnzer schon einen Struktur-Befehl?
		if(s->valid)
			throw Exception("Doppelter Befehl fr Tnzer #"+IntToStr(dancer)+" (fehlt ein \"end\"?)");

		// exec_buffer Eintrag initialisieren
		s->valid = true;
		s->cmd = cmd;

		// Hin-und her springen-Befehl
		if(cmd == "xchg")
		{
			// Strukturelementdaten aus dem Record-Puffer lesen
                        s->len = 0;
			if(sscanf(data.c_str(), "%d %d %d %d", &s->dancer, &s->pos, &s->dest, &s->reps) != 4)
				throw Exception("Ungltiger \"xchg\" Befehl");

			// Struktur visualisieren
			if(SHOW_STRUCT)
			{
				draw_arrow(s->dancer, s->pos, s->dest, clMaroon);
				draw_arrow(s->dancer, s->dest, s->pos, clMaroon);
			}
		}
		// Geschlossener Vektorzug-Befehl
		else if(cmd == "loop")
		{
			// Strukturelementdaten aus dem Record-Puffer lesen
			s->pos = 0;
			if(sscanf(data.c_str(), "%d %d %d", &s->dancer, &s->len, &s->reps) != 3)
				throw Exception("Ungltiger \"loop\" Befehl");
			if(s->moves)
				delete[] s->moves;
			s->moves = new int[2*s->len];
			for(int i = 0; i < s->len; i++)
			{
				int pos = rec_data.popi(), dest = rec_data.popi();
				*(s->moves+2*i) = pos; *(s->moves+2*i+1) = dest;

				// Struktur visualisieren
				if(SHOW_STRUCT)
					draw_arrow(s->dancer, pos, dest, clYellow);
			}
			s->reps *= s->len;

			// Ist der "loop" geschlossen?
			if(s->moves[0] != s->moves[2*s->len-1])
				throw Exception(Format("Nicht geschlossener \"loop\": %d->%d", ARRAYOFCONST((s->moves[0], s->moves[2*s->len-1]))));
		}
	}

	// Wieviele Strukturen gibt es noch in exec_buffer?
	int count = 0;

	// Alle Strukturen im Execute-Puffer ausfhren
	for(int i = 0; i < MAX_DANCERS; i++)
	{
		int pos, dest;

		// Aktuelle Struktur ermitteln;
		// falls es keine gibt, ist der Tnzer nicht sichtbar,
		// oder er kann nicht springen. Seine Sprungmglichkeiten
		// werden angezeigt.
		s = &exec_buffer[i];
		if(!s->valid)
		{
			moves_avail(dancer_pos[i]);
			plan_jump(0, -1, -1);
			continue;
		}

		count++;

		// Hin und her springen
		if(s->cmd == "xchg")
		{
			// Springen wir hin oder her?
			pos = s->pos; // Annahme: hin
			dest = s->dest;
			if(s->len++ & 1) // Annahme war falsch, wir springen zurck
			{
				pos = s->dest;
				dest = s->pos;
			}

			// Anzeigen, wie oft noch hin und her
			draw_text(s->dancer, s->pos, "x "+String(s->reps-1)+"  ");
			Application->ProcessMessages(); // Sicherstellen, dass die Zahl angezeigt wird
		}
		// Ein wiederholbarer, geschlossener Vektorzug
		else if(s->cmd == "loop")
		{
			// Sprung ermitteln...
			pos = *(s->moves+s->pos++);
			dest = *(s->moves+s->pos++);
			s->pos %= 2*s->len;

			// und visualisieren
			if(SHOW_STRUCT)
				draw_arrow(s->dancer, pos, dest, clLime);
			draw_text(s->dancer, *s->moves, "x "+String(s->reps/s->len)+"  ");
		}
		else
			throw Exception("Ungltiger Befehl in exec_buffer: \""+s->cmd+"\"");

		// Sicherstellen, dass der Sprung gltig ist
		// s->dancer ist immer ok; pos und dest werden in moves_avail geprft
		if(dancer_pos[s->dancer] != pos || !moves_avail(pos)->Find(String(dest), NULL))
			throw Exception(Format("Ungltiger Sprung in exec_buffer: %d:%d->%d", ARRAYOFCONST((s->dancer, pos, dest))));
		plan_jump(s->dancer, pos, dest);

		// Falls die Struktur fertig ausgefhrt ist, wird es als nicht gltig markiert
		if(!--s->reps)
		{
			s->valid = false;
			count--;
		}
	}

	// Falls es keine neuen Strukturen gibt, und alle Strukturen abgearbeitet sind,
	// ist der Playback vorbei.
	if(rec_data.at_end() && !count)
		state = STOP;
}
//---------------------------------------------------------------------------


// Fhre eine Simulation durch.
// Falls der Playback-Modus aktiviert ist, wird als erstens die Ausgangssituation
// (Lage der Tnzer) wiederhergestellt. In jedem Takt werden die einzelnen
// Tnzerbewegungsentscheidungen aus dem Aufnahmepuffer direkt geladen.
// Ansonsten werden die Tnzerentscheidungen in real time durchgefhrt;
// das Zielfeld wird zufllig aus der Menge der mglichen Sprngen gewhlt.
void __fastcall TForm1::GoButtonClick(TObject *Sender)
{
	// Simulation luft schon, der Benuzter wollte die Simulation beenden.
	// Weitere Clicks auf GoButton sind nicht erlaubt, bis die Simulation wirklich
	// beendet ist.
	if(state != RECORD)
	{
		state = STOP;
		GoButton->Enabled = false;
		return;
	}

	// Simulation luft jetzt.
	state = RUNNING;
	GoButton->Down = true;

	// Ausgangssituation fr den Playback wiederherstellen
	if(rec_data.data_avail())
	{
        	PLAYBACK = true;

		// Playback vorbereiten und Execute-Puffer 'leeren'
		rec_data.reset();
		for(int i = 0; i < MAX_DANCERS; i++)
			exec_buffer[i].valid = false;
		chor_data->Clear();
		clock = 0;

		// Felder wiederherstellen
		for(int i = 0; i < num_fields; i++)
			fields[i] = EMPTY;

		// Tnzerzustand wiederherstellen
		num_dancers = 0;
		for(int i = 0; i < MAX_DANCERS; i++)
		{
			int pos = dancer_pos[i] = rec_data.popi();
			// Der Tnzer ist nicht plaziert
			if(pos == -1)
				images[i]->Hide();
			// Der Tnzer ist plaziert und muss angezeigt werden
			else
			{
				// Gltigkeitsprfung
				ASSERT_VALID(i, pos, 0);
				if(fields[pos] != EMPTY)
					throw Exception("Feld #"+IntToStr(pos)+" ist doppelt besetzt");

				fields[pos] = i;

				num_dancers++;
				images[i]->Left = field_centers[pos].x - 32;
				images[i]->Top  = field_centers[pos].y - 32 + Field->Top;
				images[i]->Show();
			}
		}
	}

	// Toolbar-Knpfe aktualisieren
	idle(0, false);

	// Bis der Benuzter die Simulation stoppt (oder der Record-Puffer leer ist)...
	while(state == RUNNING)
	{
		if(PLAYBACK)
			playback();
		else
			// Tnzersprnge sequentiell bestimmen
			for(int i = 0; i < MAX_DANCERS; i++)
			{
				int dancer = i, pos = dancer_pos[dancer], dest = -1;

				// Ein Sprung aus der mglichen Sprungmenge auswhlen
				temp = moves_avail(pos);
				if(temp->Count)
					dest = temp->Strings[random(temp->Count)].ToInt();

				plan_jump(dancer, pos, dest);
			}

		// Alle Tnzer haben sich jetzt auf einen Zug festgelegt

		do_jump();
	}
}
//---------------------------------------------------------------------------

// Tnzersprnge graphisch darstellen.
// In jedem Takt, nachdem jeder Tnzer sequentiell entschieden hat,
// wohin er springt, werden alle Tnzer pseudo-gleichzeitig bewegt.
// Dies wird durch MOVE_STEPS-maliges verschieben um 1/MOVE_STEPS von dem jeweiligen
// Sprungvektor fr jeden Tnzer erreicht.
// Jeder Timer-Aufruf fhrt eine Verschiebung durch; der Timer schaltet sich nach MOVE_STEPS
// Aufrufe selber aus. Danach werden die Tnzer auf ihr Zielfeld gesetzt.
// Dies ist ntig, weil eine Integerteilung der delta-Werte zu Genauigskeitsverlust fhrt;
// der Tnzer wrde ansonsten leicht neben dem Zielfeld landen.
void __fastcall TForm1::MoveTimerTimer(TObject *Sender)
{
	// Anzahl Iterationen; grssere Zahlen => feinere Bewegung => mehr Flackern
	const int MOVE_STEPS = 10;

	MoveTimer->Tag++;

	// Alle Tnzer verschieben
	for(int i = 0; i < MAX_DANCERS; i++)
		// Falls der Sprung gltig ist,
		if(moves[i].valid)
		{
			// Tnzer i verschieben
			images[i]->Left += moves[i].dx / MOVE_STEPS;
			images[i]->Top += moves[i].dy / MOVE_STEPS;
		}

	// Timer nach MOVE_STEPS Iterationen ausschalten
	if(MoveTimer->Tag >= MOVE_STEPS)
	{
		MoveTimer->Enabled = false;

		// Tnzer genau auf ihr Feld positionieren
		for(int i = 0; i < MAX_DANCERS; i++)
		{
			TPoint center = field_centers[dancer_pos[i]];
			images[i]->Left = center.x - 32;
			images[i]->Top  = center.y - 32 + Field->Top;
		}

		// Alle Pfeile lschen
		if(SHOW_ALGO || state == RECORD)
			CLEAR_FIELD;

		// Go wieder zulassen
		if(state == STOP)
		{
			GoButton->Enabled = true;
			GoButton->Down = false;
			state = RECORD;
		}
	}
}
//---------------------------------------------------------------------------


// Gibt die Nummer des Feldes an den Pixelkoordinaten (x | y) zurck
// Algorithmus:
// Alle Mittelpunkte von den Feldern durchgehen; falls die Entfernung vom
// Mittelpunkt bis (x | y) kleiner als der Radius des Feldes ist, ist dieses
// Feld das richtige
int __fastcall TForm1::field_at_coord(int x, int y)
{
	for(int i = 0; i < num_fields; i++)
	{
		int mx = field_centers[i].x; int my = field_centers[i].y;
		int dx = x - mx; int dy = y - my;

		if(dx*dx + dy*dy <= 32*32)
			return i;
	}

	// (x | y) liegt auf keinem Feld
	return -1;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Tnzer plazieren / lschen
//---------------------------------------------------------------------------

// Plaziert bei einem Linksclick neue Tnzer aus das Feld
void __fastcall TForm1::FieldMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
	int pos = field_at_coord(X, Y);

	// Vorraussetzungen: Linksclick, Feld leer, keine laufende Simulation, noch nicht alle Tnzer plaziert
	if(Button != mbLeft || pos == -1 || fields[pos] != EMPTY || rec_data.data_avail() || num_dancers >= MAX_DANCERS)
		return;

	// Den ersten noch nicht plazierten Tnzer ermitteln
	int dancer = -1;
	while(dancer_pos[++dancer] != -1);

	// Tnzer anzeigen
	images[dancer]->Left = field_centers[pos].x - 32;
	images[dancer]->Top  = field_centers[pos].y - 32 + Field->Top;
	images[dancer]->Show();

	// Tnzerposition aktualisieren
	dancer_pos[dancer] = pos;
	fields[pos] = dancer;
	num_dancers++;
}
//---------------------------------------------------------------------------


// Ein Tnzer nach einem Rechtsclick lschen
void __fastcall TForm1::ImageMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
	// Voraussetzung: Rechtsclick, noch keine aufgenommene Daten
	if(Button != mbRight || rec_data.data_avail())
		return;

	TImage* image = dynamic_cast<TImage*>(Sender);
	int dancer = image->Tag;

	// Tnzer verstecken
	image->Hide();

	// Tnzerposition aktualisieren
	fields[dancer_pos[dancer]] = EMPTY;
	dancer_pos[dancer] = -1;
	num_dancers--;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Manuelle Verschiebung
//---------------------------------------------------------------------------

// Eine manuelle Verschiebung starten
void __fastcall TForm1::ImageStartDrag(TObject *Sender,
      TDragObject *&DragObject)
{
	int dancer = dynamic_cast<TImage*>(Sender)->Tag;
	int pos = dancer_pos[dancer];

	// Manuelle Verschiebung ist nicht mglich, wenn eine Simulation luft,
	// oder wenn der Tnzer im aktuellen Takt bereits gesprungen ist
	if(state != RECORD || PLAYBACK || moves[dancer].valid)
		CancelDrag();
	// Zeige mglichen Sprnge an
	else if(state == RECORD) // == state != RECORD && !moves[dancer].valid
		moves_avail(pos);
}
//---------------------------------------------------------------------------

// Abbruch bei einer manuellen Verschiebung - lsche die Ausgabepfeilen
void __fastcall TForm1::ImageEndDrag(TObject *Sender, TObject *Target,
      int X, int Y)
{
	// Alle Pfeile ausser den letzten Grnen lschen
	CLEAR_FIELD;
	for(int i = 0; i < MAX_DANCERS; i++)
		if(moves[i].valid)
			draw_arrow(i, moves[i].pos, moves[i].dest, clGreen);
}
//---------------------------------------------------------------------------

// Bestimmt, ob ein manuell verschobener Tnzer an der Position (x | y) abgelegt werden darf
void __fastcall TForm1::FieldDragOver(TObject *Sender, TObject *Source,
      int X, int Y, TDragState State, bool &Accept)
{
	int dest = field_at_coord(X, Y);

	// Voraussetzung: Zielfeld ist leer und Sprung ist gltig
	// ImageStartDrag() hat moves_avail in temp gespeichert
	Accept = dest != -1 && fields[dest] == EMPTY && temp->Find(IntToStr(dest), NULL);
}
//---------------------------------------------------------------------------

// Verarbeitet eine manuelle Verschiebung eines Tnzers
void __fastcall TForm1::FieldDragDrop(TObject *Sender, TObject *Source,
      int X, int Y)
{
	int dancer = dynamic_cast<TImage*>(Source)->Tag;

	int pos = dancer_pos[dancer];
	int dest = field_at_coord(X, Y);

	// Plane den Sprung
	plan_jump(dancer, pos, dest);

	// 'Takt Ausfhren' ist erlaubt
	ExecuteButton->Enabled = true;
}
//---------------------------------------------------------------------------

