//---------------------------------------------------------------------------

#ifndef mainH
#define mainH
#include "rec_data.hpp"

#include <Classes.hpp>
#include <ComCtrls.hpp>
#include <Controls.hpp>
#include <Dialogs.hpp>
#include <ExtCtrls.hpp>
#include <Graphics.hpp>
#include <ImgList.hpp>
#include <Menus.hpp>
#include <ToolWin.hpp>
#include <jpeg.hpp>
//---------------------------------------------------------------------------

// Maximale Anzahl Tnzer, Felder und Intersects
// Zur Begrndung der statischen Arrays als Datenstrukturen siehe Designentscheidungen.
const int MAX_DANCERS = 4;
const int MAX_FIELDS = 32;
const int MAX_INTS = 32;

class TForm1 : public TForm
{
	__published:	// IDE-managed Components
		TImageList *ImageList;
		TOpenDialog *OpenDialog;
		TSaveDialog *SaveDialog;
		TTimer *MoveTimer;
		TPopupMenu *ViewMenu;
			TMenuItem *ViewNone;
			TMenuItem *ViewArrows;
			TMenuItem *ViewStructures;

		TControlBar *ControlBar;
		TToolBar *ToolBar;
			TToolButton *NewButton;
			TToolButton *OpenButton;
			TToolButton *SaveButton;
			TToolButton *ToolButton1;
			TToolButton *ExecuteButton;
			TToolButton *ViewButton;
			TToolButton *GoButton;
			TToolButton *ToolButton3;
			TToolButton *AboutButton;

		TImage *Field;
		TImage *Image0;
		TImage *Image1;
		TImage *Image2;
		TImage *Image3;

		void __fastcall FormCreate(TObject *Sender);		
	 	void __fastcall NewClick(TObject *Sender);
		void __fastcall OpenClick(TObject *Sender);
		void __fastcall SaveClick(TObject *Sender);
		void __fastcall AboutClick(TObject *Sender);
		void __fastcall ViewButtonClick(TObject *Sender);
		void __fastcall ViewMenuItemClick(TObject *Sender);

		void __fastcall ExecuteButtonClick(TObject *Sender);
		void __fastcall GoButtonClick(TObject *Sender);
		void __fastcall MoveTimerTimer(TObject *Sender);

		void __fastcall FieldMouseDown(TObject *Sender, TMouseButton Button,
		  TShiftState Shift, int X, int Y);
		void __fastcall ImageMouseDown(TObject *Sender, TMouseButton Button,
		  TShiftState Shift, int X, int Y);
		void __fastcall ImageStartDrag(TObject *Sender,
		  TDragObject *&DragObject);
		void __fastcall ImageEndDrag(TObject *Sender, TObject *Target, int X,
		  int Y);
		void __fastcall FieldDragOver(TObject *Sender, TObject *Source, int X,
		  int Y, TDragState State, bool &Accept);
		void __fastcall FieldDragDrop(TObject *Sender, TObject *Source, int X,
		  int Y);

	private:
		// Tanzfeld
		int num_fields;				// Anzahl Felder auf dem Tanzfeld
		TPoint field_centers[MAX_FIELDS];	// Koordinaten der Feldermittelpunkte
		TStringList* connections[MAX_FIELDS];	// Verbindungen von Feld #x
		int fields[MAX_FIELDS];              	// Tnzer, der auf Feld #x steht
		TJPEGImage* jpeg;

		// intersect locks
		int num_intersects;			// Anzahl "Intersect Locks" auf dem Tanzfeld
		TStringList* intersects[MAX_INTS];	// Intersect locks (siehe doc)
		bool intersect_avail[MAX_INTS];		// Intersect lock #x ist verfgbar

		// Tnzerzustand
		int num_dancers;			// Anzahl Tnzer auf dem Feld
		int dancer_pos[MAX_DANCERS];		// Feld, auf der Tnzer #x steht
		TImage* images[MAX_DANCERS];		// Bilder der Tnzer
		struct					// Sprunginformationen fr jeden Tnzer
		{
			bool valid;			// Der Sprung ist in diesem Takt gltig
			int pos, dest;			// Sprung
			int dx, dy;			// Sprungvektor
		}
		moves[MAX_DANCERS];

		// Aufgenommen Sprnge / Choreographiedaten
		int clock;				// Aktuelle Taktnummer
		REC_DATA rec_data;	        	// Record-Puffer (Stack)
		TStringList* chor_data;			// Choreographie

		// Befehle zur Ausfhrung eines Strukturelements
		struct STRUCT
		{
		STRUCT():valid(false), moves(NULL) {}
			bool valid;			// Ist der Befehl gltig?
			String cmd;			// Welches Strukturelement?
			int dancer;			// Tnzernummer
			int pos, dest;			// Position und Zielfeldnummer
			int reps;			// Wiederholungszhler
			int len;			// Lnge des geschlossenen Vektorzugs
			int* moves;			// Sprnge des      "           "
		}
		exec_buffer[MAX_DANCERS];

		TStringList* temp;			// Temporre StringList fr moves_avail

		// Helferfunktionen
		void __fastcall idle(TObject *Sender, bool &Done);
		void __fastcall error(TObject *Sender, Exception *E);

		TStringList* __fastcall TForm1::moves_avail(int pos);
		int __fastcall TForm1::field_at_coord(int x, int y);

		void __fastcall TForm1::draw_arrow(int dancer, int pos, int dest, TColor color);
		void __fastcall TForm1::draw_text(int dancer, int pos, String text);
		void __fastcall TForm1::draw_moves(int pos, TStringList* moves);

		void __fastcall TForm1::plan_jump(int dancer, int pos, int dest);
		void __fastcall TForm1::do_jump();
		void __fastcall playback();

	public:
		__fastcall TForm1(TComponent* Owner):TForm(Owner){}
		__fastcall ~TForm1()
		{
			for(int i = 0; i < MAX_DANCERS; i++)
				if(exec_buffer[i].moves)
					delete[] exec_buffer[i].moves;
		}
};
//---------------------------------------------------------------------------

extern PACKAGE TForm1 *Form1;
//---------------------------------------------------------------------------

#endif
